@@ -6,30 +6,30 @@
 
 #ifdef HAVE_CACHE
 
+#include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
 #include <pthread.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include "libfs.h"
 #include "session.h"
 #include "cache.h"
 
-#define MAX_DELAY_TIMER 10
-#define MAX_INDEX 27 * 27
+#define MAX_CACHE_DELAY_TIMER 10
+#define MAX_CACHE_INDEX 27 * 27
 
-static t_cached_object *cache[MAX_INDEX];
-static pthread_mutex_t cache_mutex[MAX_INDEX];
-static volatile unsigned long long cachesize;
+static t_cached_object *cache[MAX_CACHE_INDEX];
+static pthread_mutex_t cache_mutex[MAX_CACHE_INDEX];
+static volatile off_t cachesize;
 static int delay_timer = 0;
 
 void init_cache_module(void) {
 	int i;
 
-	for (i = 0; i < MAX_INDEX; i++) {
+	for (i = 0; i < MAX_CACHE_INDEX; i++) {
 		cache[i] = NULL;
 		pthread_mutex_init(&cache_mutex[i], NULL);
 	}
@@ -104,7 +104,7 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	struct stat status;
 	off_t size;
 	int fd, i;
-	ssize_t bytes_read, bytes_total = 0;
+	size_t bytes_read, bytes_total = 0;
 
 	if ((size = filesize(file)) == -1) {
 		return NULL;
@@ -226,13 +226,13 @@ void check_cache(void) {
 	t_cached_object *object;
 	int i;
 
-	if (++delay_timer == MAX_DELAY_TIMER) {
-		for (i = 0; i < MAX_INDEX; i++) {
+	if (++delay_timer == MAX_CACHE_DELAY_TIMER) {
+		for (i = 0; i < MAX_CACHE_INDEX; i++) {
 			pthread_mutex_lock(&cache_mutex[i]);
 
 			object = cache[i];
 			while (object != NULL) {
-				if ((object->timer -= MAX_DELAY_TIMER) <= 0) {
+				if ((object->timer -= MAX_CACHE_DELAY_TIMER) <= 0) {
 					if (object->busy <= 0) {
 						object = remove_from_cache(object, i);
 						continue;
@@ -250,4 +250,75 @@ void check_cache(void) {
 	}
 }
 
+#ifdef HAVE_COMMAND
+void print_cache_list(FILE *fp) {
+	t_cached_object *object;
+	int i, files = 0, time;
+	off_t size = 0;
+
+	for (i = 0; i < MAX_CACHE_INDEX; i++) {
+		pthread_mutex_lock(&cache_mutex[i]);
+
+		object = cache[i];
+		while (object != NULL) {
+			fprintf(fp, "  Filename : %s\n", object->file);
+			fprintf(fp, "  File size: %lld bytes\n", (long long)object->size);
+			if ((time = object->timer - delay_timer) > 0) {
+				fprintf(fp, "  Time left: %d seconds\n\n", time);
+			} else {
+				fprintf(fp, "  File marked for removal from cache\n\n");
+			}
+			files++;
+			size = size + object->size;
+			object = object->next;
+		}
+
+		pthread_mutex_unlock(&cache_mutex[i]);
+	}
+
+	fprintf(fp, "  Total: %lld bytes in %d files.\n", (long long)size, files);
+}
+
+int clear_cache(void) {
+	t_cached_object *object, *list;
+	int i, removed = 0;
+
+	for (i = 0; i < MAX_CACHE_INDEX; i++) {
+		pthread_mutex_lock(&cache_mutex[i]);
+
+		list = cache[i];
+		cache[i] = NULL;
+		while (list != NULL) {
+			object = list;
+			list = list->next;
+
+			if (object->busy == 0) {
+				/* Object unused, so remove
+				 */
+				cachesize -= object->size;
+				
+				free(object->data);
+				free(object->file);
+				free(object);
+
+				removed++;
+			} else {
+				/* Object busy, put back in list
+				 */
+				object->next = cache[i];
+				cache[i] = object;
+			}
+		}
+
+		pthread_mutex_unlock(&cache_mutex[i]);
+	}
+
+	return removed;
+}
+
+off_t size_of_cache(void) {
+	return cachesize;
+}
+#endif
+
 #endif
