@@ -7,7 +7,7 @@
 typedef struct type_cached_object {
 	char          *file;
 	char          *data;
-	unsigned long size;
+	off_t         size;
 	int           timer;
 	time_t        last_changed;
 	volatile int  busy;
@@ -22,5 +22,10 @@ t_cached_object *add_to_cache(t_session *session, char *file);
 t_cached_object *search_cache(t_session *session, char *file);
 void done_with_cached_object(t_cached_object *object);
 void check_cache(void);
+#ifdef HAVE_COMMAND
+void print_cache_list(FILE *fp);
+int clear_cache(void);
+off_t size_of_cache(void);
+#endif
 
 #endif
