@@ -3,6 +3,7 @@
  * All the routines for handling CGI requests.
  */
 #include "config.h"
+#include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -10,7 +11,6 @@
 #include <netdb.h>
 #include <errno.h>
 #include <pthread.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #ifdef HAVE_NETINET_IN_H
@@ -22,8 +22,9 @@
 #include "envir.h"
 #include "log.h"
 
-#define MAX_DELAY_TIMER 60
+#define MAX_CGI_DELAY_TIMER 60
 #define ip_index(ip) (ip & 255)
+#define DUMMY_BUFFER_SIZE 65536
 
 static int delay_timer = 0;
 
@@ -112,7 +113,7 @@ void check_load_balancer(t_config *config) {
 	t_connect_to  *connect_to = NULL;
 	int i, sock;
 
-	if (++delay_timer == MAX_DELAY_TIMER) {
+	if (++delay_timer == MAX_CGI_DELAY_TIMER) {
 		fcgi_server = config->fcgi_server;
 		while (fcgi_server != NULL) {
 			/* Check session timeouts
@@ -125,7 +126,7 @@ void check_load_balancer(t_config *config) {
 				while (cgi_session != NULL) {
 					next = cgi_session->next;
 
-					if (((cgi_session->timer -= MAX_DELAY_TIMER) <= 0) || (cgi_session->connect_to->available == false)) {
+					if (((cgi_session->timer -= MAX_CGI_DELAY_TIMER) <= 0) || (cgi_session->connect_to->available == false)) {
 						if (last == NULL) {
 							fcgi_server->cgi_session_list[i] = next;
 						} else {
@@ -170,18 +171,28 @@ void check_load_balancer(t_config *config) {
 
 t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *cgi_script) {
 	char *extension;
+	t_fcgi_server *fcgi;
+	int i;
 
-	extension = file_extension(cgi_script);
-	while (fcgi_server != NULL) {
-		if (in_charlist(fcgi_server->fcgi_id, fastcgi)) {
-			if (in_charlist(extension, &(fcgi_server->extension))) {
-				break;
-			}
-		}
-		fcgi_server = fcgi_server->next;
+	if ((fastcgi == NULL) || (cgi_script == NULL)) {
+		return NULL;
 	}
 
-	return fcgi_server;
+	extension = file_extension(cgi_script);
+
+	for (i = 0; i < fastcgi->size; i++) {
+		fcgi = fcgi_server;
+		while (fcgi != NULL) {
+			if (strcmp(fastcgi->item[i], fcgi->fcgi_id) == 0) {
+				if (in_charlist(extension, &(fcgi_server->extension))) {
+					return fcgi;
+				}
+			}
+			fcgi = fcgi->next;
+		}
+	}
+
+	return NULL;
 }
 
 
@@ -190,9 +201,10 @@ t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi
  * =====================
  */
 
-int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
-	int cgi_pid, post_pipe[2], html_pipe[2], error_pipe[2], i;
+pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
+	int post_pipe[2], html_pipe[2], error_pipe[2], i;
 	char *slash, *run[7], cgi_time[16];
+	pid_t cgi_pid;
 
 	do {
 		if (pipe(post_pipe) != -1) {
@@ -226,18 +238,12 @@ int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			close(error_pipe[0]);
 			close(error_pipe[1]);
 
-			fcntl(0, F_SETFD, 0);
-			fcntl(1, F_SETFD, 0);
-			fcntl(2, F_SETFD, 0);
+			fcntl(STDIN_FILENO, F_SETFD, 0);
+			fcntl(STDOUT_FILENO, F_SETFD, 0);
+			fcntl(STDERR_FILENO, F_SETFD, 0);
 
 			set_environment(session, NULL);
-/*
-#ifdef HAVE_SSL
-			if (session->binding->use_ssl) {
-				ssl_set_environment(session->ssl_data);
-			}
-#endif
-*/
+
 			if ((slash = strrchr(session->file_on_disk, '/')) != NULL) {
 				*slash = '\0';
 				chdir(session->file_on_disk);
@@ -247,8 +253,8 @@ int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			i = 0;
 			if (cgi_info->wrap_cgi) {
 				run[i++] = session->config->cgi_wrapper;
-				if (session->host->cgi_wrap_id != NULL) {
-					setenv("CGIWRAP_ID", session->host->cgi_wrap_id, 1);
+				if (session->host->wrap_cgi != NULL) {
+					setenv("CGIWRAP_ID", session->host->wrap_cgi, 1);
 				} else {
 					setenv("CGIWRAP_ID", session->local_user, 1);
 				}
@@ -261,7 +267,7 @@ int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 				snprintf(cgi_time, 15, "%d", session->config->time_for_cgi);
 				setenv("CGIWRAP_TIMEFORCGI", cgi_time, 1);
 			} else if (setsid() == -1) {
-				exit(-1);
+				exit(EXIT_FAILURE);
 			}
 			if (session->cgi_handler != NULL) {
 				run[i++] = session->cgi_handler;
@@ -276,7 +282,7 @@ int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			run[i] = NULL;
 
 			execvp(run[0], run + 1);
-			exit(-1);
+			exit(EXIT_FAILURE);
 		default:
 			/* Parent, reads CGI output
 			 */
@@ -307,9 +313,7 @@ int fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	bool read_again;
 	fd_set read_fds;
-	int bytes_read;
-
-	cgi_info->input_result = cgi_info->error_result = cd_NOTHING;
+	int bytes_read, result = cgi_OKE;
 
 	do {
 		read_again = false;
@@ -320,12 +324,16 @@ int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 
 		switch (select(cgi_info->highest_fd, &read_fds, NULL, NULL, &(cgi_info->select_timeout))) {
 			case -1:
-				return cr_ERROR;
+				if (errno != EINTR) {
+					return cgi_ERROR;
+				} else {
+					read_again = true;
+				}
 			case 0:
 				if (session->force_quit) {
-					return cr_FORCE_QUIT;
+					return cgi_FORCE_QUIT;
 				} else if (--(cgi_info->timer) <= 0) {
-					return cr_TIMEOUT;
+					return cgi_TIMEOUT;
 				} else {
 					cgi_info->select_timeout.tv_sec = 1;
 					cgi_info->select_timeout.tv_usec = 0;
@@ -334,50 +342,47 @@ int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 		}
 	} while (read_again);
 
-	do {
+	/* Read CGI output
+	 */
+	if (FD_ISSET(cgi_info->from_cgi, &read_fds)) do {
 		read_again = false;
 
-		if (FD_ISSET(cgi_info->from_cgi, &read_fds)) {
-			bytes_read = read(cgi_info->from_cgi, cgi_info->input_buffer + cgi_info->input_len, cgi_info->input_buffer_size - cgi_info->input_len);
-			if (bytes_read == -1) {
-				if (errno != EINTR) {
-					cgi_info->input_result = cd_READ_ERROR;
-				} else {
-					read_again = true;
-				}
-			} else if (bytes_read == 0) {
-				cgi_info->input_result = cd_END_OF_DATA;
-			} else if (bytes_read > 0) {
-				cgi_info->input_result = cd_READ_DATA;
-				cgi_info->input_len += bytes_read;
+		bytes_read = read(cgi_info->from_cgi, cgi_info->input_buffer + cgi_info->input_len, cgi_info->input_buffer_size - cgi_info->input_len);
+		if (bytes_read == -1) {
+			if (errno != EINTR) {
+				return cgi_ERROR;
+			} else {
+				read_again = true;
 			}
+		} else if (bytes_read == 0) {
+			result = cgi_END_OF_DATA;
+		} else if (bytes_read > 0) {
+			cgi_info->input_len += bytes_read;
 		}
 	} while (read_again);
 
-	do {
+	/* Read CGI error output
+	 */
+	if (FD_ISSET(cgi_info->cgi_error, &read_fds)) do {
 		read_again = false;
 
-		if (FD_ISSET(cgi_info->cgi_error, &read_fds)) {
-			bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
-			if (bytes_read == -1) {
-				if (errno != EINTR) {
-					cgi_info->error_result = cd_READ_ERROR;
-				} else {
-					read_again = true;
-				}
-			} else if (bytes_read == 0) {
-				cgi_info->error_result = cd_END_OF_DATA;
-			} else if (bytes_read > 0) {
-				cgi_info->error_result = cd_READ_DATA;
-				cgi_info->error_len += bytes_read;
+		bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
+		if (bytes_read == -1) {
+			if (errno != EINTR) {
+				return cgi_ERROR;
+			} else {
+				read_again = true;
 			}
+//		} else if (bytes_read == 0) {
+//			result = cgi_END_OF_DATA;
+		} else if (bytes_read > 0) {
+			cgi_info->error_len += bytes_read;
 		}
 	} while (read_again);
 
-	return cr_OKE;
+	return result;
 }
 
-
 /*
  *  Fast CGI processes
  *  ===================
@@ -408,7 +413,7 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 	return sock;
 }
 
-int	send_fcgi_data(t_session *session, int sock) {
+int	send_fcgi_request(t_session *session, int sock) {
 	t_fcgi_buffer fcgi_buffer;
 
 	fcgi_buffer.sock = sock;
@@ -439,29 +444,28 @@ int	send_fcgi_data(t_session *session, int sock) {
 }
 
 static int read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
-	int bytes_read = -1;
+	int bytes_read, result = cgi_ERROR;
 	fd_set read_fds;
-	struct timeval select_timeout;
 	bool read_again;
 
-	select_timeout.tv_sec = 1;
-	select_timeout.tv_usec = 0;
-
 	do {
+		read_again = false;
+
 		FD_ZERO(&read_fds);
 		FD_SET(cgi_info->from_cgi, &read_fds);
 
-		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &select_timeout)) {
+		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &(cgi_info->select_timeout))) {
 			case -1:
-				return cr_ERROR;
+				break;
 			case 0:
 				if (session->force_quit) {
-					return cr_FORCE_QUIT;
+					result = cgi_FORCE_QUIT;
 				} else if (--(cgi_info->timer) <= 0) {
-					return cr_TIMEOUT;
+					result = cgi_TIMEOUT;
 				} else {
-					select_timeout.tv_sec = 1;
-					select_timeout.tv_usec = 0;
+					cgi_info->select_timeout.tv_sec = 1;
+					cgi_info->select_timeout.tv_usec = 0;
+					read_again = true;
 				}
 				break;
 			default:
@@ -469,121 +473,132 @@ static int read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buff
 					read_again = false;
 					if ((bytes_read = read(cgi_info->from_cgi, buffer, size)) == -1) {
 						if (errno != EINTR) {
-							return cr_ERROR;
+							return cgi_ERROR;
 						} else {
 							read_again = true;
 						}
 					}
 				} while (read_again);
+				result = bytes_read;
 		}
-	} while (bytes_read < 0);
+	} while (read_again);
 
-	return bytes_read;
+	return result;
 }
 
 int read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
-	char *buffer;
+	char *buffer, dummy[DUMMY_BUFFER_SIZE];
 	bool read_again;
-	unsigned char header[8];
-	int bytes_read, size, padding, offset = 0, bytes_left;
-
-	cgi_info->input_result = cgi_info->error_result = cd_NOTHING;
+	int bytes_read, bytes_left;
+	unsigned int content, padding, data_length;
 
 	/* Read header
 	 */
-	size = 8;
-	do {
-		read_again = false;
+	if (cgi_info->read_header) {
+		bytes_left = FCGI_HEADER_LENGTH;
+		do {
+			read_again = false;
 
-		switch (bytes_read = read_fcgi_socket(session, cgi_info, (char*)header + 8 - size, size)) {
-			case cr_TIMEOUT:
-			case cr_FORCE_QUIT:
-				return bytes_read;
-			case cr_ERROR:
-				if (errno != EINTR) {
-					cgi_info->input_result = cd_READ_ERROR;
-					return cr_OKE;
-				} else {
-					read_again = true;
-				}
-				break;
-			case 0:
-				cgi_info->input_result = cd_END_OF_DATA;
-				return cr_OKE;
-			default:
-				if ((size -= bytes_read) == 0) {
-					if (header[1] == FCGI_END_REQUEST) {
-						cgi_info->input_result = cd_END_OF_DATA;
-						return cr_OKE;
+			switch (bytes_read = read_fcgi_socket(session, cgi_info, cgi_info->header + FCGI_HEADER_LENGTH - bytes_left, bytes_left)) {
+				case cgi_TIMEOUT:
+					return cgi_TIMEOUT;
+				case cgi_FORCE_QUIT:
+					return cgi_FORCE_QUIT;
+				case cgi_ERROR:
+					if (errno != EINTR) {
+						return cgi_ERROR;
+					} else {
+						read_again = true;
 					}
-				} else {
-					read_again = true;
-				}
-		}
-	} while (read_again);
+					break;
+				case 0:
+					return cgi_END_OF_DATA;
+				default:
+					if ((bytes_left -= bytes_read) > 0) {
+						read_again = true;
+					}
+			}
+		} while (read_again);
+
+		cgi_info->read_header = false;
+		cgi_info->fcgi_data_len = 0;
+	}
 
 	/* Determine the size and type of the data
 	 */
-	padding = (int)header[6];
-	size = 256 * (int)header[4] + (int)header[5] + padding;
-	switch (header[1]) {
+	content = 256 * (unsigned char)cgi_info->header[4] + (unsigned char)cgi_info->header[5];
+	padding = (unsigned char)cgi_info->header[6];
+	data_length = content + padding;
+
+	switch ((unsigned char)cgi_info->header[1]) {
 		case FCGI_STDOUT:
-			buffer = cgi_info->input_buffer;
+			buffer = cgi_info->input_buffer + cgi_info->input_len;
+			bytes_left = cgi_info->input_buffer_size - cgi_info->input_len;
 			break;
 		case FCGI_STDERR:
-			buffer = cgi_info->error_buffer;
+			buffer = cgi_info->error_buffer + cgi_info->error_len;
+			bytes_left = cgi_info->error_buffer_size - cgi_info->error_len;
 			break;
 		case FCGI_END_REQUEST:
-			cgi_info->input_result = cd_END_OF_DATA;
-			return cr_OKE;
+			return cgi_END_OF_DATA;
 		default:
-			lseek(cgi_info->from_cgi, size, SEEK_CUR);
-			return cr_OKE;
+			/* Unsupported type, so skip data
+			 */
+			buffer = dummy;
+			bytes_left = DUMMY_BUFFER_SIZE;
 	}
 
-	/* Read data
-	 */
-	bytes_left = size;
-	do {
-		read_again = false;
+	if (data_length > 0) {
+		/* Read data
+		 */
+		do {
+			if (bytes_left > (data_length - cgi_info->fcgi_data_len)) {
+				bytes_left = data_length - cgi_info->fcgi_data_len;
+			}
+			read_again = false;
 
-		switch (bytes_read = read_fcgi_socket(session, cgi_info, buffer + offset, bytes_left)) {
-			case cr_TIMEOUT:
-			case cr_FORCE_QUIT:
-				return bytes_read;
-			case cr_ERROR:
-				if (errno != EINTR) {
-					cgi_info->input_result = cd_READ_ERROR;
-					return cr_OKE;
-				} else {
-					read_again = true;
-				}
-				break;
-			case 0:
-				if (size == 0) {
-					cgi_info->input_result = cd_END_OF_DATA;
-				}
-				return cr_OKE;
-			default:
-				offset += bytes_read;
-				if ((bytes_left -= bytes_read) > 0) {
-					read_again = true;
-				}
-		}
-	} while (read_again);
-
-	switch (header[1]) {
-		case FCGI_STDOUT:
-			cgi_info->input_result = cd_READ_DATA;
-			cgi_info->input_len = size - padding;
-			break;
-		case FCGI_STDERR:
-			cgi_info->error_result = cd_READ_DATA;
-			cgi_info->error_len = size - padding;
-			break;
-		default:
-			break;
+			switch (bytes_read = read_fcgi_socket(session, cgi_info, buffer, bytes_left)) {
+				case cgi_TIMEOUT:
+					return cgi_TIMEOUT;
+				case cgi_FORCE_QUIT:
+					return cgi_FORCE_QUIT;
+				case cgi_ERROR:
+					if (errno != EINTR) {
+						return cgi_ERROR;
+					} else {
+						read_again = true;
+					}
+					break;
+				case 0:
+					return cgi_END_OF_DATA;
+				default:
+					if ((cgi_info->fcgi_data_len += bytes_read) == data_length) {
+						cgi_info->read_header = true;
+					}
+					if (cgi_info->fcgi_data_len > content) {
+						/* Read more then content (padding)
+						 */
+						if (cgi_info->fcgi_data_len - bytes_read < content) {
+							bytes_read = content - (cgi_info->fcgi_data_len - bytes_read);
+						} else {
+							bytes_read = 0;
+						}
+					}
+					switch ((unsigned char)cgi_info->header[1]) {
+						case FCGI_STDOUT:
+							cgi_info->input_len += bytes_read;
+							break;
+						case FCGI_STDERR:
+							cgi_info->error_len += bytes_read;
+							break;
+						default:
+							break;
+					}
+			}
+		} while (read_again);
+	} else {
+		cgi_info->read_header = true;
 	}
 
-	return cr_OKE;
+	return cgi_OKE;
 }
