@@ -1,13 +1,14 @@
 #include "config.h"
+#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 #include <dirent.h>
 #include <pthread.h>
-#include <sys/types.h>
 #include <sys/wait.h>
 #include "client.h"
+#include "command.h"
 #include "log.h"
 
 #define ip_index(ip) (ip & 255)
@@ -74,9 +75,9 @@ int add_client(t_session *session) {
 
 /* Remember the clientrecord for flooding prevention
  */
-void mark_client_for_removal(t_session *session, int delay) {
+int mark_client_for_removal(t_session *session, int delay) {
 	t_client *list;
-	int i;
+	int i, result = 0;
 
 	i = ip_index(session->ip_address);
 
@@ -87,21 +88,28 @@ void mark_client_for_removal(t_session *session, int delay) {
 		if (list->session == session) {
 			close_socket(list->session);
 			list->delayed_remove = delay - 1;
+			result = 1;
 			break;
 		}
 		list = list->next;
 	}
 
 	pthread_mutex_unlock(&client_mutex[i]);
+
+	return result;
 }
 
 /* Check the delayed_remove timers and remove client
  * when timer has reached 0
  */
-void check_delayed_remove_timers(void) {
+void check_delayed_remove_timers(t_config *config) {
 	t_client *client, *prev = NULL, *next;
 	int i;
 
+	if (config->reconnect_delay <= 0) {
+		return;
+	}
+
 	for (i = 0; i < 256; i++) {
 		pthread_mutex_lock(&client_mutex[i]);
 
@@ -134,7 +142,7 @@ void check_delayed_remove_timers(void) {
 
 /* Remove a client from the client_list.
  */
-void remove_client(t_session *session, bool free_session) {
+int remove_client(t_session *session, bool free_session) {
 	t_client *to_be_removed = NULL, *list;
 	int i;
 
@@ -167,6 +175,10 @@ void remove_client(t_session *session, bool free_session) {
 			free(to_be_removed->session);
 		}
 		free(to_be_removed);
+
+		return 1;
+	} else {
+		return 0;
 	}
 }
 
@@ -226,10 +238,10 @@ int connection_allowed(long ip, int maxperip, int maxtotal) {
 
 /* Disconnect all connected clients.
  */
-void disconnect_clients(t_config *config) {
+int disconnect_clients(t_config *config) {
 	t_client *client;
 	t_directory *dir;
-	int max_wait = 30, i;
+	int max_wait = 30, i, kicked = 0;
 
 	for (i = 0; i < 256; i++) {
 		pthread_mutex_lock(&client_mutex[i]);
@@ -238,6 +250,7 @@ void disconnect_clients(t_config *config) {
 		while (client != NULL) {
 			client->session->force_quit = true;
 			client = client->next;
+			kicked++;
 		}
 
 		pthread_mutex_unlock(&client_mutex[i]);
@@ -254,12 +267,15 @@ void disconnect_clients(t_config *config) {
 		dir->nr_of_clients = 0;
 		dir = dir->next;
 	}
+	
+	return kicked;
 }
 
 /* Kick an IP address.
  */
-void kick_ip(long ip) {
+int kick_ip(long ip) {
 	t_client *client;
+	int result = 0;
 
 	pthread_mutex_lock(&client_mutex[ip_index(ip)]);
 
@@ -267,11 +283,14 @@ void kick_ip(long ip) {
 	while (client != NULL) {
 		if (client->session->ip_address == ip) {
 			client->session->force_quit = true;
+			result++;
 		}
 		client = client->next;
 	}
 
 	pthread_mutex_unlock(&client_mutex[ip_index(ip)]);
+
+	return result;
 }
 
 /* Check if the client is flooding the server with requests
@@ -284,50 +303,11 @@ bool client_is_flooding(t_session *session) {
 	return ((session->kept_alive * session->config->flooding_time) > (session->config->flooding_count * time_passed));
 }
 
-#ifdef HAVE_COMMAND
-/* Extra client functions
- */
-void print_client_list(FILE *fp) {
-	t_client *client;
-	char *host;
-	unsigned char *ip_address;
-	int i;
-
-	for (i = 0; i < 256; i++) {
-		pthread_mutex_lock(&client_mutex[i]);
-
-		client = client_list[i];
-		while (client != NULL) {
-			fprintf(fp, "session id  : %d\n", client->session->client_id);
-			ip_address = (unsigned char*)&(client->session->ip_address);
-			fprintf(fp, "IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
-			fprintf(fp, "socket      : %d\n", client->session->client_socket);
-			fprintf(fp, "Kept alive  : %d\n", client->session->kept_alive);
-			if (client->session->remote_user != NULL) {
-			fprintf(fp, "Remote user : %s\n", client->session->remote_user);
-			}
-			if (client->session->file_on_disk != NULL) {
-				fprintf(fp, "File on disk: %s\n", client->session->file_on_disk);
-				if ((host = get_headerfield("Host:", client->session->headerfields)) != NULL) {
-					fprintf(fp, "Host        : %s\n", host);
-				}
-			}
-			client = client->next;
-			if (client != NULL) {
-				fprintf(fp, "---\n");
-			}
-		}
-
-		pthread_mutex_unlock(&client_mutex[i]);
-	}
-}
-#endif
-
 /* Disconnect a client.
  */
-void kick_client(int id) {
+int kick_client(int id) {
 	t_client *client;
-	int i;
+	int i, result = 0;
 
 	for (i = 0; i < 256; i++) {
 		pthread_mutex_lock(&client_mutex[i]);
@@ -336,6 +316,7 @@ void kick_client(int id) {
 		while (client != NULL) {
 			if (client->session->client_id == id) {
 				client->session->force_quit = true;
+				result = 1;
 				break;
 			}
 			client = client->next;
@@ -343,6 +324,8 @@ void kick_client(int id) {
 
 		pthread_mutex_unlock(&client_mutex[i]);
 	}
+
+	return result;
 }
 
 /* IP ban functions
@@ -372,8 +355,13 @@ int ban_ip(long ip, int timer, bool kick_on_ban) {
 			ban->bantime = timer;
 			ban->connect_attempts = 0;
 			ban->next = banlist;
-
 			banlist = ban;
+
+#ifdef HAVE_COMMAND
+			increment_counter(COUNTER_BAN);
+#endif
+
+			retval = 1;
 		} else {
 			retval = -1;
 		}
@@ -381,8 +369,8 @@ int ban_ip(long ip, int timer, bool kick_on_ban) {
 
 	pthread_mutex_unlock(&ban_mutex);
 
-	if (kick_on_ban) {
-		kick_ip(ip);
+	if (kick_on_ban && new_ip) {
+		retval = kick_ip(ip);
 	}
 
 	return retval;
@@ -390,8 +378,9 @@ int ban_ip(long ip, int timer, bool kick_on_ban) {
 
 /* Reset the timer of a banned IP address.
  */
-void reban_ip(long ip) {
+int reban_ip(long ip) {
 	t_banned *ban;
+	int result = 0;
 
 	pthread_mutex_lock(&ban_mutex);
 
@@ -399,17 +388,20 @@ void reban_ip(long ip) {
 	while (ban != NULL) {
 		if (ban->ip == ip) {
 			ban->timer = ban->bantime;
+			result = 1;
 			break;
 		}
 		ban = ban->next;
 	}
 
 	pthread_mutex_unlock(&ban_mutex);
+	
+	return result;
 }
 
 /* Check the timers of the banlist.
  */
-void check_banlist(t_config *config) {
+void check_ban_list(t_config *config) {
 	t_banned *ban, *prev = NULL, *next;
 
 	pthread_mutex_lock(&ban_mutex);
@@ -442,13 +434,15 @@ void check_banlist(t_config *config) {
 
 /* Unban an IP address.
  */
-void unban_ip(long ip) {
-	t_banned *ban, *prev = NULL;
+int unban_ip(long ip) {
+	t_banned *ban, *prev = NULL, *next;
+	int result = 0;
 
 	pthread_mutex_lock(&ban_mutex);
 	
 	ban = banlist;
 	while (ban != NULL) {
+		next = ban->next;
 		if ((ban->ip == ip) || (ip == 0)) {
 			if (prev == NULL) {
 				banlist = ban->next;
@@ -456,35 +450,109 @@ void unban_ip(long ip) {
 				prev->next = ban->next;
 			}
 			free(ban);
-			break;
+			result++;
+
+			if (ip != 0) {
+				break;
+			}
+		} else {
+			prev = ban;
 		}
-		prev = ban;
-		ban = ban->next;
+		ban = next;
 	}
 
 	pthread_mutex_unlock(&ban_mutex);
+
+	return result;
 }
 
 #ifdef HAVE_COMMAND
+/* Print the list of current connections.
+ */
+void print_client_list(FILE *fp) {
+	t_client *client;
+	unsigned char *ip_address;
+	int i, count = 0;
+
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
+
+		client = client_list[i];
+		while (client != NULL) {
+			fprintf(fp, "  Client ID   : %d\n", client->session->client_id);
+			ip_address = (unsigned char*)&(client->session->ip_address);
+			fprintf(fp, "  IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
+			fprintf(fp, "  Socket      : %d\n", client->session->client_socket);
+			if (client->session->remote_user != NULL) {
+				fprintf(fp, "  Remote user : %s\n", client->session->remote_user);
+			}
+			fprintf(fp, "  Kept alive  : %d\n\n", client->session->kept_alive);
+			client = client->next;
+			count++;
+		}
+
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
+
+	fprintf(fp, "  Total: %d clients\n", count);
+}
+
+int number_of_clients(void) {
+	int result = 0, i;
+	t_client *client;
+
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
+
+		client = client_list[i];
+		while (client != NULL) {
+			result++;
+			client = client->next;
+		}
+
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
+
+	return result;
+}
+
 /* Print the list of banned IP addresses.
  */
-void print_banlist(FILE *fp) {
+void print_ban_list(FILE *fp) {
 	t_banned *ban;
 	unsigned char *ip_address;
+	int count = 0;
 
 	pthread_mutex_lock(&ban_mutex);
 
 	ban = banlist;
 	while (ban != NULL) {
 		ip_address = (unsigned char*)&(ban->ip);
-		fprintf(fp, "IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
-		fprintf(fp, "seconds left: %d\n", ban->timer);
+		fprintf(fp, "  IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
+		fprintf(fp, "  seconds left: %d\n\n", ban->timer);
 		ban = ban->next;
-		if (ban != NULL) {
-			fprintf(fp, "---\n");
-		}
+		count++;
 	}
+	fprintf(fp, "  Total: %d bans\n", count);
 
 	pthread_mutex_unlock(&ban_mutex);
 }
+
+int number_of_bans(void) {
+	t_banned *ban;
+	int result = 0;
+
+	pthread_mutex_lock(&ban_mutex);
+	
+	ban = banlist;
+	while (ban != NULL) {
+		result++;
+		ban = ban->next;
+	}
+
+	pthread_mutex_unlock(&ban_mutex);
+
+	return result;
+}
+
 #endif
