@@ -1,6 +1,7 @@
 #ifndef _CLIENT_H
 #define _CLIENT_H
 
+#include "config.h"
 #include <stdbool.h>
 #include "serverconfig.h"
 #include "session.h"
@@ -11,26 +12,27 @@
 
 void init_client_module(void);
 int  add_client(t_session *session);
-void mark_client_for_removal(t_session *session, int delay);
-void check_delayed_remove_timers(void);
-void remove_client(t_session *session, bool free_session);
+int  mark_client_for_removal(t_session *session, int delay);
+void check_delayed_remove_timers(t_config *config);
+int  remove_client(t_session *session, bool free_session);
 int  connection_allowed(long ip, int maxperip, int maxtotal);
-void disconnect_clients(t_config *config);
+int  disconnect_clients(t_config *config);
 bool client_is_flooding(t_session *session);
 void check_flooding(t_config *config);
 
-#ifdef HAVE_COMMAND
-void print_clientlist(FILE *fp);
-#endif
-void kick_client(int id);
-void kick_ip(long ip);
+int  kick_client(int id);
+int  kick_ip(long ip);
 
 int  ban_ip(long ip, int timer, bool kick_on_ban);
-void reban_ip(long ip);
-void check_banlist(t_config *config);
-void unban_ip(long ip);
+int  reban_ip(long ip);
+void check_ban_list(t_config *config);
+int  unban_ip(long ip);
+
 #ifdef HAVE_COMMAND
-void print_banlist(FILE *fp);
+void print_client_list(FILE *fp);
+int number_of_clients(void);
+void print_ban_list(FILE *fp);
+int number_of_bans(void);
 #endif
 
 #endif
