@@ -1,4 +1,5 @@
 #include "config.h"
+#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -9,7 +10,9 @@
 #include <dirent.h>
 #include <errno.h>
 #include <pthread.h>
+#include <syslog.h>
 #include <grp.h>
+#include <netdb.h>
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -18,9 +21,9 @@
 #endif
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include "alternative.h"
 #include "mimetype.h"
 #include "serverconfig.h"
 #include "libfs.h"
@@ -50,7 +53,6 @@
 #include "cache.h"
 #endif
 
-#define FILENAME_BUFFER_SIZE   256
 #define REQUEST_BUFFER_CHUNK  4096
 #define MAIN_LOOP_TIMER          1
 
@@ -64,13 +66,35 @@
 #define rs_QUIT_SERVER           1
 #define rs_UNBAN_CLIENTS         2
 #define rs_TOUCH_LOGFILES        3
+			
+typedef struct {
+	char *config_dir;
+	bool daemon;
+	bool config_check;
+} t_settings;
 
 volatile int received_signal = rs_NONE;
-
-char *default_configdir      = CONFIG_DIR;
-char *main_configfile        = "httpd.conf";
-
-char *hs_conlen = "Content-Length: ";
+char *hs_conlen      = "Content-Length: ";
+char *fb_filesystem  = "access denied via filesystem";
+char *fb_symlink     = "symlink not allowed";
+char *fb_accesslist  = "access denied via accesslist";
+char *version_string = "Hiawatha v"VERSION
+#ifdef HAVE_SSL
+			", SSL"
+#endif
+#ifdef HAVE_CACHE
+			", cache"
+#endif
+#ifdef HAVE_COMMAND
+			", CommandChannel"
+#endif
+#ifdef HAVE_PLUGIN
+			", plugin"
+#endif
+#ifdef DEBUG
+			", debug"
+#endif
+;
 
 #ifdef DEBUG
 
@@ -81,27 +105,27 @@ double t1, t2;
 struct timeval tp;
 
 void start_clock(char *mesg) {
-	fprintf(stderr, "(%5d) START: %s\n", getpid(), mesg);
+	fprintf(stderr, "(%5d) START: %s\n", (int)getpid(), mesg);
 	gettimeofday(&tp, NULL);
-	t1 = (double)tp.tv_sec+(1.e-6)*tp.tv_usec;
+	t1 = (double)tp.tv_sec + (1.e-6)*tp.tv_usec;
 }
 
 void stop_clock(void) {
 	gettimeofday(&tp, NULL);
-	t2 = (double)tp.tv_sec+(1.e-6)*tp.tv_usec;
-	fprintf(stderr, "(%5d)  time: %f\n", getpid(), t2 - t1);
+	t2 = (double)tp.tv_sec + (1.e-6)*tp.tv_usec;
+	fprintf(stderr, "(%5d)  time: %f\n", (int)getpid(), t2 - t1);
 }
 
 #endif
 
 /* Create all logfiles with the right ownership and accessrights
  */
-void touch_logfiles(t_config *config, char *basedir) {
+void touch_logfiles(t_config *config, char *dir) {
 	t_host *host;
 
-	touch_logfile(basedir, config->system_logfile, 0640, config->server_uid, config->server_gid);
+	touch_logfile(dir, config->system_logfile, 0640, config->server_uid, config->server_gid);
 	if (config->garbage_logfile != NULL) {
-		touch_logfile(basedir, config->garbage_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, config->garbage_logfile, 0640, config->server_uid, config->server_gid);
 	}
 
 	host = config->first_host;
@@ -109,8 +133,8 @@ void touch_logfiles(t_config *config, char *basedir) {
 		if (host->access_fileptr != NULL) {
 			fflush(host->access_fileptr);
 		}
-		touch_logfile(basedir, host->access_logfile, 0640, config->server_uid, config->server_gid);
-		touch_logfile(basedir, host->error_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, host->access_logfile, 0640, config->server_uid, config->server_gid);
+		touch_logfile(dir, host->error_logfile, 0640, config->server_uid, config->server_gid);
 		host = host->next;
 	}
 }
@@ -153,7 +177,7 @@ int handle_error(t_session *session) {
 		if (session->host->return_code != -1) {
 			session->return_code = session->host->return_code;
 		}
-		retval = run_script(session);
+		retval = execute_cgi(session);
 	} else {
 		retval = send_file(session);
 	}
@@ -183,13 +207,10 @@ int handle_request(t_session *session) {
 		}
 	}
 
-	if (strcmp(session->method, "HEAD") == 0) {
-		session->head_request = true;
-	}
 	if ((strcmp(session->method, "GET") == 0) || session->head_request) {
 		if (session->is_cgi_script) {
 			session->body = NULL;
-			retval = run_script(session);
+			retval = execute_cgi(session);
 		} else {
 			retval = send_file(session);
 		}
@@ -200,7 +221,7 @@ int handle_request(t_session *session) {
 		}
 	} else if (strcmp(session->method, "POST") == 0) {
 		if (session->is_cgi_script) {
-			retval = run_script(session);
+			retval = execute_cgi(session);
 		} else {
 			retval = 405;
 		}
@@ -262,6 +283,7 @@ int fetch_request(t_session *session) {
 						if ((strend = strstr(strstart, "\r\n")) != NULL) {
 							*strend = '\0';
 							if ((content_length = str2int(strstart)) == -1) {
+								result = 400;
 								break;
 							}
 							session->content_length = content_length;
@@ -347,6 +369,10 @@ int fetch_request(t_session *session) {
 		}
 	} while (keep_reading);
 
+#ifdef HAVE_COMMAND
+	increment_transfer(TRANSFER_RECEIVED, session->header_length + session->content_length);
+#endif
+
 	return result;
 }
 
@@ -358,7 +384,7 @@ int parse_request(t_session *session, int total_bytes) {
 
 	request_end = session->request + total_bytes;
 
-	/* Scan the first request line.
+	/* Request method
 	 */
 	session->method = str_end = session->request;
 	while ((*str_end != ' ') && (str_end != request_end)) {
@@ -367,10 +393,11 @@ int parse_request(t_session *session, int total_bytes) {
 	if (str_end == request_end) {
 		return 400;
 	}
-
 	*str_end = '\0';
 	session->uri = ++str_end;
 
+	/* URI
+	 */
 	while ((*str_end != ' ') && (str_end != request_end)) {
 		str_end++;
 		if (*str_end == '?') {
@@ -386,6 +413,9 @@ int parse_request(t_session *session, int total_bytes) {
 
 	*str_end = '\0';
 	str_end++;
+
+	/* Protocol version
+	 */
 	if (strlen(str_end) < 8) {
 		return 400;
 	} else if (memcmp(str_end, "HTTP/", 5) != 0) {
@@ -394,9 +424,12 @@ int parse_request(t_session *session, int total_bytes) {
 
 	session->http_version = str_end;
 	str_end += 7;
-	if ((session->body = strstr(str_end + 1, "\r\n\r\n")) != NULL) {
-		*(session->body + 2) = '\0';
-		session->body += 4;
+
+	/* Body and other request headerlines
+	 */
+	if (session->content_length > 0) {
+		session->body = session->request + session->header_length;
+		*(session->body - 2) = '\0';
 	}
 	if (str_end + 5 != session->body) {
 		session->headerfields = parse_headerfields(str_end + 3);
@@ -441,79 +474,93 @@ int parse_request(t_session *session, int total_bytes) {
 		session->kept_alive++;
 	}
 
+	if (strcmp(session->method, "HEAD") == 0) {
+		session->head_request = true;
+	}
+
 	return retval;
 }
 
 /* Convert the request uri to a filename.
  */
 int uri_to_path(t_session *session) {
-	int retval = 200, length, alias_length;
+	size_t length, alias_length = 0;
 	char *strstart, *strend;
 	t_keyvalue *alias;
+	int retval;
 	
 	/* Requested file in userdirectory?
 	 */
-	if ((session->host->user_websites) && (*(session->uri + 1) == '~')) {
-		strstart = session->uri + 1;
-		if ((strend = strchr(strstart, '/')) == NULL) {
-			return 301;
-		} else if ((session->local_user = (char*)malloc(strend - strstart + 1)) == NULL) {
-			return 500;
-		}
-		memcpy(session->local_user, strstart, strend - strstart);
-		*(session->local_user + (strend - strstart)) = '\0';
-		
-		if (duplicate_host(session)) {
-			retval = get_homedir(session, session->local_user + 1);
-			session->host->error_handler = NULL;
-		} else {
-			retval = 500;
-		}
-	}
+	if ((session->host->user_websites) && (strlen(session->uri) > 3)) {
+		if (*(session->uri + 1) == '~') {
+			strstart = session->uri + 1;
+			if ((strend = strchr(strstart, '/')) == NULL) {
+				return 301;
+			} else if ((length = strend - strstart) > 1) {
+				if ((session->local_user = (char*)malloc(length + 1)) == NULL) {
+					return 500;
+				}
 
-	/* Convert request to complete path + filename.
-	 */
-	if (retval == 200) {
-		if ((session->file_on_disk = (char*)malloc(FILENAME_BUFFER_SIZE + 4)) != NULL) { // + 3 for .gz (gzip encoding)
-			*(session->file_on_disk + FILENAME_BUFFER_SIZE) = '\0';
-
-			/* Search for an alias.
-			 */
-			alias = session->host->alias;
-			while (alias != NULL) {
-				alias_length = strlen(alias->key);
-				if (strncmp(session->uri, alias->key, alias_length) == 0) {
-					if ((*(session->uri + alias_length) == '/') ||
-						(*(session->uri + alias_length) == '\0')) {
-						break;
+				memcpy(session->local_user, strstart, length);
+				*(session->local_user + length) = '\0';
+				
+				if (duplicate_host(session)) {
+					if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
+						return retval;
 					}
+					session->host->error_handler = NULL;
+				} else {
+					return 500;
 				}
-				alias = alias->next;
-			}
-			if (alias == NULL) {
-				/* No alias
-				 */
-				strncpy(session->file_on_disk, session->host->website_root, FILENAME_BUFFER_SIZE);
-				strstart = session->uri;
-				if (session->local_user != NULL) {
-					strstart += strlen(session->local_user) + 1;
-				}
-				length = strlen(session->file_on_disk);
-				strncpy(session->file_on_disk + length, strstart, FILENAME_BUFFER_SIZE - length);
 			} else {
-				/* Use alias
-				 */
-				strncpy(session->file_on_disk, alias->value, FILENAME_BUFFER_SIZE);
-				length = strlen(session->file_on_disk);
-				strncpy(session->file_on_disk + length, session->uri + alias_length, FILENAME_BUFFER_SIZE - length);
+				// uri is '/~/'
+				return 404;
 			}
-
-		} else {
-			retval = 500;
 		}
 	}
 
-	return retval;
+	/* Search for an alias.
+	 */
+	alias = session->host->alias;
+	while (alias != NULL) {
+		alias_length = strlen(alias->key);
+		if (strncmp(session->uri, alias->key, alias_length) == 0) {
+			if ((*(session->uri + alias_length) == '/') || (*(session->uri + alias_length) == '\0')) {
+				break;
+			}
+		}
+		alias = alias->next;
+	}
+
+	/* Allocate memory
+	 */
+	if (alias == NULL) {
+		length = strlen(session->host->website_root);
+	} else {
+		length = strlen(alias->value);
+	}
+	length += strlen(session->uri) + strlen(session->host->start_file);
+	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { // + 3 for '.gz' (gzip encoding)
+		return 500;
+	}
+
+	/* Copy stuff
+	 */
+	if (alias == NULL) {
+		length = strlen(session->host->website_root);
+		memcpy(session->file_on_disk, session->host->website_root, length);
+		strstart = session->uri;
+		if (session->local_user != NULL) {
+			strstart += strlen(session->local_user) + 1;
+		}
+	} else {
+		length = strlen(alias->value);
+		memcpy(session->file_on_disk, alias->value, length);
+		strstart = session->uri + alias_length;
+	}
+	strcpy(session->file_on_disk + length, strstart);
+
+	return 200;
 }
 
 #ifdef HAVE_PLUGIN
@@ -552,7 +599,7 @@ t_access allow_client(t_session *session) {
  */
 int serve_client(t_session *session) {
 	int result, length;
-	char *dir_config, *dir, *search, *hostname;
+	char *search, *hostname, *conffile;
 	t_host *host_record;
 
 	if ((result = fetch_request(session)) != 200) {
@@ -561,9 +608,9 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	parse_special_chars(session->uri, strlen(session->uri) + 1);
+	parse_special_chars(session->uri, strlen(session->uri));
 	if (session->vars != NULL) {
-		parse_special_chars(session->vars, strlen(session->vars) + 1);
+		parse_special_chars(session->vars, strlen(session->vars));
 	}
 
 	session->host = session->config->first_host;
@@ -573,46 +620,49 @@ int serve_client(t_session *session) {
 		}
 	}
 
-	if ((result = uri_to_path(session)) != 200) {
+	if (session->host->require_resolve_ip && (session->ip_resolved == false)) {
+		/* See if the client IP-address is resolvable
+		 */
+		if (gethostbyaddr((unsigned char*)&(session->ip_address), 4, AF_INET) == NULL) {
+			log_error(session, "can't resolve IP");
+			return 403;
+		}
+		session->ip_resolved = true;
+	}
+
+	if (valid_uri(session->uri) == false) {
+		return 404;
+	} else if ((result = uri_to_path(session)) != 200) {
 		return result;
 	}
 
 	/* Load configfile from directories
 	 */
-	if ((dir_config = strdup(session->file_on_disk)) != NULL) {
-		search = dir_config;
-		while (*search != '\0') {
-			if (*search == '/') {
-				length = search - dir_config + 1;
-				if ((dir = (char*)malloc(length + 16)) != NULL) {
-					memcpy(dir, dir_config, length);
-					memcpy(dir + length, ".hiawatha\0", 10);
-					if (duplicate_host(session)) {
-						if (read_user_configfile(dir, session->host) == -1) {
-							result = 500;
-						}
-					} else {
+	search = session->file_on_disk;
+	while (*search != '\0') {
+		if (*search == '/') {
+			length = search - session->file_on_disk + 1;
+			if ((conffile = (char*)malloc(length + 10)) != NULL) {
+				memcpy(conffile, session->file_on_disk, length);
+				memcpy(conffile + length, ".hiawatha\0", 10);
+				if (duplicate_host(session)) {
+					if (read_user_configfile(conffile, session->host) > 0) {
+						log_file_error(session, conffile, "error in configuration file");
 						result = 500;
 					}
-					free(dir);
 				} else {
 					result = 500;
 				}
-			}
-
-			if (result == 200) {
-				search++;
+				free(conffile);
 			} else {
-				break;
+				result = 500;
 			}
 		}
-		free(dir_config);
-
-		if (result != 200) {
+		if (result == 200) {
+			search++;
+		} else {
 			return result;
 		}
-	} else {
-		return 500;
 	}
 
 #ifdef HAVE_SSL
@@ -622,10 +672,8 @@ int serve_client(t_session *session) {
 	}
 #endif
 
-	if (valid_uri(session->uri) == false) {
-		return 404;
-	} else if (client_is_rejected_bot(session)) {
-		set_403_reason(session, fb_DENYBOT);
+	if (client_is_rejected_bot(session)) {
+		log_error(session, "bot rejected");
 		return 403;
 	}
 
@@ -633,7 +681,7 @@ int serve_client(t_session *session) {
 	if (is_plugin_uri(session)) {
 		switch (allow_client(session)) {
 			case deny:
-				set_403_reason(session, fb_ACCESSLIST);
+				log_error(session, fb_accesslist);
 				return 403;
 				break;
 			case allow:
@@ -649,6 +697,10 @@ int serve_client(t_session *session) {
 	}
 #endif
 
+	if ((result = copy_directory_settings(session)) != 200) {
+		return result;
+	}
+
 	switch (is_directory(session->file_on_disk)) {
 		case error:
 			return 500;
@@ -658,21 +710,24 @@ int serve_client(t_session *session) {
 		case no:
 			break;
 		case no_access:
-			set_403_reason(session, fb_FILESYSTEM);
+			log_error(session, fb_filesystem);
 			return 403;
 		case not_found:
-			if ((search = get_headerfield("Accept-Encoding:", session->headerfields)) != NULL) {
-				if ((strstr(search, "gzip")) != NULL) {
-					session->accept_gzip = true;
+			if (session->host->use_gz_file) {
+				if ((search = get_headerfield("Accept-Encoding:", session->headerfields)) != NULL) {
+					if ((strstr(search, "gzip")) != NULL) {
+						session->accept_gzip = true;
+					}
 				}
 			}
+			if (session->accept_gzip == false) {
+				return 404;
+			}
 	}
 
-	if ((result = copy_directory_settings(session)) != 200) {
-		return result;
-	} else switch (allow_client(session)) {
+	switch (allow_client(session)) {
 		case deny:
-			set_403_reason(session, fb_ACCESSLIST);
+			log_error(session, fb_accesslist);
 			return 403;
 			break;
 		case allow:
@@ -682,12 +737,15 @@ int serve_client(t_session *session) {
 			if (http_authentication_oke(session) == false) {
 				return 401;
 			}
-			break;
 	}
 
-	if (*(session->uri + strlen(session->uri) - 1) == '/') {
-		length = strlen(session->file_on_disk);
-		strncpy(session->file_on_disk + length, session->host->start_file, FILENAME_BUFFER_SIZE - length);
+	length = strlen(session->file_on_disk);
+	if (*(session->file_on_disk + length - 1) == '/') {
+		if (session->uri_is_dir) {
+			strcpy(session->file_on_disk + length, session->host->start_file);
+		} else {
+			return 404;
+		}
 	} else if (session->uri_is_dir) {
 		return 301;
 	}
@@ -710,7 +768,7 @@ void handle_request_result(t_session *session, int result) {
 			send_code(session, 400);
 			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_max_request_size, session->config->kick_on_ban);
-				log_warning(session->config->system_logfile, "Client banned because of sending a too large request", session->ip_address);
+				log_system(session, "Client banned because of sending a too large request");
 			}
 			break;
 		case fe_TIMEOUT:
@@ -718,39 +776,39 @@ void handle_request_result(t_session *session, int result) {
 				send_code(session, 408);
 				if ((session->config->ban_on_timeout > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 					ban_ip(session->ip_address, session->config->ban_on_timeout, session->config->kick_on_ban);
-					log_warning(session->config->system_logfile, "Client banned because of connection timeout", session->ip_address);
+					log_system(session, "Client banned because of connection timeout");
 				} else {
-					log_warning(session->config->system_logfile, "Timeout while waiting for request", session->ip_address);
+					log_system(session, "Timeout while waiting for request");
 				}
 			}
 			break;
 		case fe_CLIENT_DISCONNECTED:
 			if (session->kept_alive == 0) {
-				log_warning(session->config->system_logfile, "Client disconnected", session->ip_address);
+				log_system(session, "Client disconnected");
 			}
 			break;
 		case fe_READ_ERROR:
 			if (errno != ECONNRESET) {
-				log_warning(session->config->system_logfile, "Error while reading request", session->ip_address);
+				log_system(session, "Error while reading request");
 #ifdef DEBUG
-				log_warning(session->config->system_logfile, strerror(errno), session->ip_address);
+				log_system(session, strerror(errno));
 #endif
 			}
 			break;
 		case fe_FORCE_QUIT:
-			log_warning(session->config->system_logfile, "Client kicked", session->ip_address);
+			log_system(session, "Client kicked");
 			break;
 		case rr_CMD_INJECTION:
 			log_request(session, result);
 			if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_cmdi, session->config->kick_on_ban);
-				log_warning(session->config->system_logfile, "Client banned because of command injection", session->ip_address);
+				log_system(session, "Client banned because of command injection");
 			}
 		case rr_SQL_INJECTION:
 			log_request(session, result);
 			if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_sqli, session->config->kick_on_ban);
-				log_warning(session->config->system_logfile, "Client banned because of SQL injection", session->ip_address);
+				log_system(session, "Client banned because of SQL injection");
 			}
 			send_code(session, 400);
 			break;
@@ -767,7 +825,7 @@ void handle_request_result(t_session *session, int result) {
 			send_code(session, result);
 			if ((session->config->ban_on_garbage > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_garbage, session->config->kick_on_ban);
-				log_warning(session->config->system_logfile, "Client banned because of sending garbage", session->ip_address);
+				log_system(session, "Client banned because of sending garbage");
 			}
 			break;
 		case 500:
@@ -813,19 +871,13 @@ void connection_handler(t_session *session) {
 			handle_request_result(session, result);
 			send_buffer(session, NULL, 0); // Flush the output-buffer
 
-#ifdef HAVE_COMMAND
-			CC_client_lock();
-#endif
 			reset_session(session);
-#ifdef HAVE_COMMAND
-			CC_client_unlock();
-#endif
 
 			if (session->keep_alive && (session->config->ban_on_flooding > 0)) {
 				if (client_is_flooding(session)) {
 					if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
 						ban_ip(session->ip_address, session->config->ban_on_flooding, session->config->kick_on_ban);
-						log_warning(session->config->system_logfile, "Client banned because of flooding", session->ip_address);
+						log_system(session, "Client banned because of flooding");
 						session->keep_alive = false;
 					}
 				}
@@ -837,31 +889,31 @@ void connection_handler(t_session *session) {
 		close(session->client_socket);
 	}
 	log_close(session->config->first_host);
-	//destroy_session(session);
 
-#ifdef HAVE_COMMAND
-	CC_client_lock();
-#endif
 	if (session->config->reconnect_delay > 0) {
 		mark_client_for_removal(session, session->config->reconnect_delay);
 	} else {
 		remove_client(session, true);
 	}
-#ifdef HAVE_COMMAND
-	CC_client_unlock();
-#endif
 	// Client session ends here.
 }
 
-void TERM_handler() {
+/* Signal handlers
+ */
+void SEGV_handler(int sig) {
+	syslog(LOG_DAEMON | LOG_ALERT, "segmentation fault!");
+	exit(EXIT_FAILURE);
+}
+
+void TERM_handler(int sig) {
 	received_signal = rs_QUIT_SERVER;
 }
 
-void USR1_handler() {
+void USR1_handler(int sig) {
 	received_signal = rs_UNBAN_CLIENTS;
 }
 
-void USR2_handler() {
+void USR2_handler(int sig) {
 	received_signal = rs_TOUCH_LOGFILES;
 }
 
@@ -883,6 +935,7 @@ int fill_read_fds(fd_set *read_fds, t_binding *binding, t_binding *bind_command)
 		}
 		binding = binding->next;
 	}
+
 #ifdef HAVE_COMMAND
 	while (bind_command != NULL) {
 		FD_SET(bind_command->socket, read_fds);
@@ -907,7 +960,7 @@ int fill_read_fds(fd_set *read_fds, t_binding *binding, t_binding *bind_command)
 
 /* Create a socketlist.
  */
-int bind_socket(t_binding *binding) {
+int bind_sockets(t_binding *binding) {
 	int one;
 	struct sockaddr_in saddr;
 	char *ip_address;
@@ -946,7 +999,7 @@ int bind_socket(t_binding *binding) {
 /* Accept or deny an incoming connection.
  */
 int accept_connection(t_binding *binding, t_config *config) {
-	unsigned int       size;
+	socklen_t          size;
 	bool               kick_client;
 	t_session          *session;
 	struct sockaddr_in caddr;
@@ -997,18 +1050,21 @@ int accept_connection(t_binding *binding, t_config *config) {
 			if ((config->ban_on_max_per_ip > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, config->ban_on_max_per_ip, config->kick_on_ban);
 			}
-			log_warning(config->system_logfile, "Maximum number of connections for IP address reached", session->ip_address);
+			log_system(session, "Maximum number of connections for IP address reached");
 			break;
 		case ca_TOOMUCH_TOTAL:
-			log_warning(config->system_logfile, "Maximum number of total connections reached", session->ip_address);
+			log_system(session, "Maximum number of total connections reached");
 			break;
 		case ca_BANNED:
 #ifdef DEBUG
-			log_warning(config->system_logfile, "Client kicked because of ban", session->ip_address);
+			log_system(session, "Client kicked because of ban");
 #endif
 			if (config->reban_during_ban && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				reban_ip(session->ip_address);
 			}
+#ifdef HAVE_COMMAND
+			increment_counter(COUNTER_DENY);
+#endif
 			break;
 	}
 
@@ -1020,29 +1076,12 @@ int accept_connection(t_binding *binding, t_config *config) {
 	return 0;
 }
 
-#ifndef HAVE_CLEARENV
-void clearenv(void) {
-	extern char **environ;
-	char *eq, key[256];
-	int len;
-
-	while (*environ != NULL) {
-		if ((eq = strchr(*environ, '=')) != NULL) {
-			if ((len = eq - *environ) < 256) {
-				memcpy(key, *environ, len);
-				key[len] = '\0';
-				unsetenv(key);
-			}
-		}
-		environ++;
-	}
-}
-#endif
-
 /* Run the Hiawatha webserver.
  */
-int run_server(char *configdir, bool daemon) {
-	int                quit = 0, result, highest_fd;
+int run_server(t_settings *settings) {
+	char               *main_configfile = "httpd.conf";
+	int                highest_fd;
+	bool               must_quit = false;
 #ifdef HAVE_COMMAND
 	t_admin            *admin;
 	struct sockaddr_in caddr;
@@ -1050,46 +1089,57 @@ int run_server(char *configdir, bool daemon) {
 #endif
 	pid_t              pid;
 	t_binding          *binding;
-	FILE               *fp;
 	t_config           *config;
     fd_set             read_fds;
 	struct timeval     select_timeout;
 
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGCHLD, SIG_IGN);
+	signal(SIGTSTP, SIG_IGN);
+	signal(SIGABRT, SIG_IGN);
+	signal(SIGQUIT, SIG_IGN);
+	signal(SIGSEGV, SEGV_handler);
 
 	config = default_config();
-	chdir(configdir);
-	if ((result = read_main_configfile(main_configfile, config)) == 0) {
-		result = check_configuration(config);
+	if (chdir(settings->config_dir) == -1) {
+		perror(settings->config_dir);
+		return EXIT_FAILURE;
+	} else if (settings->config_check) {
+		printf("Using %s\n", settings->config_dir);
 	}
-	if (result != 0) {
-		print_configerror(main_configfile, result);
-		return -1;
-	}
-	if (read_mimetypes(config->mimetype_config, &(config->mimetype)) == -1) {
-		printf("Error while reading mimetype configuration.\n");
-		return -1;
-	}
-	if (read_throttleconfig(config->throttle_config, &(config->throttle)) == -1) {
-		//printf("Error while reading throttle configuration.\n");
-		//return -1
+	if (read_main_configfile(main_configfile, config, settings->config_check) == -1) {
+		return EXIT_FAILURE;
+	} else if (check_configuration(config) == -1) {
+		return EXIT_FAILURE;
 	}
 
-	/* Bind Serverport
+	if (read_mimetypes(config->mimetype_config, &(config->mimetype)) == -1) {
+		fprintf(stderr, "Error while reading mimetype configuration.\n");
+		return EXIT_FAILURE;
+	}
+	if (read_throttleconfig(config->throttle_config, &(config->throttle)) == -1) {
+		fprintf(stderr, "Error while reading throttle configuration.\n");
+		return EXIT_FAILURE;
+	}
+
+	if (settings->config_check) {
+		printf("Configuration oke.\n");
+		return EXIT_SUCCESS;
+	}
+	
+	/* Bind Serverports
 	 */
-	if (bind_socket(config->binding) == -1) {
-		return -1;
+	if (bind_sockets(config->binding) == -1) {
+		return EXIT_FAILURE;
 	}
 
 #ifdef HAVE_SSL
-	sslInit();
+	ssl_init();
 	binding = config->binding;
 	while (binding != NULL) {
 		if (binding->use_ssl) {
-			if ((binding->ssl_context = ssl_binding(binding->server_key, "")) == NULL) {
-				printf("Error while reading keyfile %s!\n", binding->server_key);
-				return -1;
+			if ((binding->ssl_context = ssl_binding(binding->server_key, binding->ca_cert, binding->verify_depth, config->dh_file, config->allowed_ciphers)) == NULL) {
+				return EXIT_FAILURE;
 			}
 		}
 		binding = binding->next;
@@ -1097,8 +1147,10 @@ int run_server(char *configdir, bool daemon) {
 #endif
 
 #ifdef HAVE_COMMAND
-	if (bind_socket(config->command_port) == -1) {
-		return -1;
+	/* Bind CommandChannels
+	 */
+	if (bind_sockets(config->command_port) == -1) {
+		return EXIT_FAILURE;
 	}
 #endif
 
@@ -1110,8 +1162,8 @@ int run_server(char *configdir, bool daemon) {
 			}
 			break;
 		default:
-			printf("Error while initializing plugin.\n");
-			return -1;
+			fprintf(stderr, "Error while initializing plugin.\n");
+			return EXIT_FAILURE;
 	}
 #endif
 
@@ -1121,29 +1173,26 @@ int run_server(char *configdir, bool daemon) {
 
 	/* Become a daemon
 	 */
-	if (daemon) {
+	if (settings->daemon) {
 		switch (pid = fork()) {
 			case -1:
 				perror("fork()");
-				return -1;
+				return EXIT_FAILURE;
 			case 0:
 				if (setsid() == -1) {
 					perror("setsid()");
-					return -1;
+					return EXIT_FAILURE;
 				}
 				break;
 			default:
-				if ((fp = fopen(config->pidfile, "w")) != NULL) {
-					fprintf(fp, "%d\n", pid);
-					fclose(fp);
-				} else {
-					printf("Warning: can't write PID file %s.\n", config->pidfile);
-				}
-				return 0;
+				log_pid(config, pid);
+				return EXIT_SUCCESS;
 		}
+	} else {
+		log_pid(config, getpid());
 	}
 
-	/* Security
+	/* Change server root
 	 */
 	if (config->server_root != NULL) {
 		do {
@@ -1152,13 +1201,15 @@ int run_server(char *configdir, bool daemon) {
 					break;
 				}
 			}
-			printf("\nError while changing root to %s!\n", config->server_root);
-			return -1;
+			fprintf(stderr, "\nError while changing root to %s!\n", config->server_root);
+			return EXIT_FAILURE;
 		} while (false);
 	} else {
 		chdir("/");
 	}
 
+	/* Change userid
+	 */
 	if (getuid() == 0) do {
 		if (setgroups(config->groups.number, config->groups.array) != -1) {
 			if (setgid(config->server_gid) != -1) {
@@ -1167,19 +1218,28 @@ int run_server(char *configdir, bool daemon) {
 				}
 			}
 		}
-		printf("\nError while changing uid/gid!\n");
-		return -1;
+		fprintf(stderr, "\nError while changing uid/gid!\n");
+		return EXIT_FAILURE;
 	} while (false);
 
+	if (settings->daemon == false) {
+		printf("Press Ctrl-C to shutdown the Hiawatha webserver.\n");
+		signal(SIGINT, TERM_handler);
+	} else {
+		signal(SIGINT, SIG_IGN);
+	}
+
 	signal(SIGTERM, TERM_handler);
 	signal(SIGUSR1, USR1_handler);
 	signal(SIGUSR2, USR2_handler);
 
+	/* Start listening for incoming connections
+	 */
 	binding = config->binding;
 	while (binding != NULL) {
 		if (listen(binding->socket, 16) == -1) {
-			perror("listen()");
-			return -1;
+			perror("http(s) socket");
+			return EXIT_FAILURE;
 		}
 		binding = binding->next;
 	}
@@ -1187,8 +1247,8 @@ int run_server(char *configdir, bool daemon) {
 	binding = config->command_port;
 	while (binding != NULL) {
 		if (listen(binding->socket, 1) == -1) {
-			perror("listen()");
-			return -1;
+			perror("commandchannel socket");
+			return EXIT_FAILURE;
 		}
 		binding = binding->next;
 	}
@@ -1218,9 +1278,6 @@ int run_server(char *configdir, bool daemon) {
 		switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
-#ifdef DEBUG
-					log_error(config->system_logfile, errno);
-#endif
 					log_string(config->system_logfile, "Fatal error selecting connection");
 					usleep(1000);
 				}
@@ -1229,10 +1286,8 @@ int run_server(char *configdir, bool daemon) {
 				select_timeout.tv_sec = MAIN_LOOP_TIMER;
 				select_timeout.tv_usec = 0;
 
-				check_banlist(config);
-				if (config->reconnect_delay > 0) {
-					check_delayed_remove_timers();
-				}
+				check_ban_list(config);
+				check_delayed_remove_timers(config);
 				check_load_balancer(config);
 #ifdef HAVE_CACHE
 				check_cache();
@@ -1250,7 +1305,7 @@ int run_server(char *configdir, bool daemon) {
 #ifdef DEBUG
 						log_string(config->system_logfile, "Received TERM signal");
 #endif
-						quit = 1;
+						must_quit = true;
 						break;
 					case rs_UNBAN_CLIENTS:
 						unban_ip(0);
@@ -1268,19 +1323,8 @@ int run_server(char *configdir, bool daemon) {
 				admin = first_admin();
 				while (admin != NULL) {
 					if (FD_ISSET(admin->socket, &read_fds)) {
-						switch (handle_admin(admin, config)) {
-							case cc_DISCONNECT:
-								remove_admin(admin->socket);
-								break;
-							case cc_SHUTDOWN:
-#ifdef DEBUG
-								log_string(config->system_logfile, "Shutdown by admin");
-#endif
-								quit = 1;
-								break;
-							case cc_UNBAN_CLIENTS:
-								received_signal = rs_UNBAN_CLIENTS;
-								break;
+						if (handle_admin(admin, config) == cc_DISCONNECT) {
+							remove_admin(admin->socket);
 						}
 					}
 					admin = next_admin();
@@ -1323,7 +1367,9 @@ int run_server(char *configdir, bool daemon) {
 				}
 #endif
 		}
-	} while (quit == 0);
+	} while (must_quit == false);
+
+	signal(SIGTERM, SIG_DFL);
 
 	disconnect_clients(config);
 #ifdef HAVE_COMMAND
@@ -1349,48 +1395,63 @@ int run_server(char *configdir, bool daemon) {
 	}
 #endif
 
-	signal(SIGTERM, SIG_DFL);
-	signal(SIGUSR1, SIG_DFL);
-	signal(SIGUSR2, SIG_DFL);
-
 	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
 
 #ifdef HAVE_PLUGIN
 	plugin_destroy();
 #endif
 
-	return 0;
+	return EXIT_SUCCESS;
 }
 
-void show_help(char *hiabin) {
-	printf("Usage %s: [-c path] [-d] [-h]\n", hiabin);
-	printf("\t-c path: path to where the configrationfiles are located.\n");
-	printf("\t-d: don't fork to the background.\n");
-	printf("\t-h: show this information.\n");
+void show_help(char *hiawatha) {
+	printf("Usage: %s [options]\n", hiawatha);
+	printf("Options: -c <path>: path to where the configrationfiles are located.\n");
+	printf("         -d: don't fork to the background.\n");
+	printf("         -h: show this information and exit.\n");
+	printf("         -k: check configuration and exit.\n");
+	printf("         -v: show version and compile information and exit.\n");
 }
 
 /* Main and stuff...
  */
 int main(int argc, char *argv[]) {
 	int i = 0;
-	char *configdir = default_configdir;
-	bool daemon = true;
+	t_settings settings;
 
+	/* Default settings
+	 */
+	settings.config_dir   = CONFIG_DIR;
+	settings.daemon       = true;
+	settings.config_check = false;
+
+	/* Read commandline parameters
+	 */
 	while (++i < argc) {
 		if (strcmp(argv[i], "-c") == 0) {
 			if (++i < argc) {
-				configdir = argv[i];
+				settings.config_dir = argv[i];
+			} else {
+				fprintf(stderr, "Specify a directory.\n");
+				return EXIT_FAILURE;
 			}
 		} else if (strcmp(argv[i], "-d") == 0) {
-			daemon = false;
+			settings.daemon = false;
 		} else if (strcmp(argv[i], "-h") == 0) {
 			show_help(argv[0]);
-			return 0;
+			return EXIT_SUCCESS;
+		} else if (strcmp(argv[i], "-k") == 0) {
+			settings.config_check = true;
+		} else if (strcmp(argv[i], "-v") == 0) {
+			printf("%s\n", version_string);
+			return EXIT_SUCCESS;
 		} else {
-			printf("Unknown option\n");
-			return -1;
+			fprintf(stderr, "Unknown option. Use '-h' for help.\n");
+			return EXIT_FAILURE;
 		}
 	}
 
-	return run_server(configdir, daemon);
+	/* Run Hiawatha
+	 */
+	return run_server(&settings);
 }
