@@ -19,7 +19,7 @@
 static bool group_oke(t_session *session, char *user) {
 	bool retval;
 	FILE *gfp;
-	char *pwfile, *line, *item, *rest;
+	char *pwfile, line[257], *item, *rest;
 	bool groupfound;
 	int i;
 
@@ -27,32 +27,29 @@ static bool group_oke(t_session *session, char *user) {
 		retval = false;
 		pwfile = file_in_chroot(session, session->host->groupfile);
 		if ((gfp = fopen_neighbour(pwfile, "r", session->file_on_disk)) != NULL) {
-			if ((line = (char*)malloc(257)) != NULL) {
-				*(line + 256) = '\0';
-				while (fgets(line, 256, gfp) != NULL) {
-					if (split_string(line, &item, &rest, ':') == 0) {
-						groupfound = false;
-						for (i = 0; i < session->host->required_group.size; i++) {
-							if (strcmp(*(session->host->required_group.item + i), item) == 0) {
-								groupfound = true;
+			line[256] = '\0';
+			while (fgets(line, 256, gfp) != NULL) {
+				if (split_string(line, &item, &rest, ':') == 0) {
+					groupfound = false;
+					for (i = 0; i < session->host->required_group.size; i++) {
+						if (strcmp(*(session->host->required_group.item + i), item) == 0) {
+							groupfound = true;
+							break;
+						}
+					}
+					if (groupfound) {
+						while (rest != NULL) {
+							split_string(rest, &item, &rest, ' ');
+							if (strcmp(user, item) == 0) {
+								retval = true;
 								break;
 							}
 						}
-						if (groupfound) {
-							while (rest != NULL) {
-								split_string(rest, &item, &rest, ' ');
-								if (strcmp(user, item) == 0) {
-									retval = true;
-									break;
-								}
-							}
-						}
-					}
-					if (retval == true) {
-						break;
 					}
 				}
-				free(line);
+				if (retval == true) {
+					break;
+				}
 			}
 			fclose(gfp);
 		}
@@ -66,13 +63,13 @@ static bool group_oke(t_session *session, char *user) {
 /* Read a password from a file
  */
 static char *get_password(t_session *session, char *username) {
-	char *pwfile, line[256], *result = NULL, *sep;
+	char *pwfile, line[257], *result = NULL, *sep;
 	FILE *pfp;
 	
 	pwfile = file_in_chroot(session, session->host->passwordfile);
 	if ((pfp = fopen_neighbour(pwfile, "r", session->file_on_disk)) != NULL) {
-		line[255] = '\0';
-		while ((sep = fgets(line, 255, pfp)) != NULL) {
+		line[256] = '\0';
+		while ((sep = fgets(line, 256, pfp)) != NULL) {
 			while ((*sep != ':') && (*sep != '\0')) {
 				sep++;
 			}
@@ -127,8 +124,7 @@ bool basic_http_authentication(t_session *session, char *auth_str) {
 					/* Password match?
 					 */
 					if (strcmp(encrypted, passwd) == 0) {
-						session->remote_user = strdup(auth_user);
-						retval = true;
+						retval = ((session->remote_user = strdup(auth_user)) != NULL);
 					}
 				}
 				free(passwd);
