@@ -1,14 +1,14 @@
 #include "config.h"
+#include <sys/types.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
-#include <stdbool.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include "libfs.h"
 #include "libstr.h"
 
@@ -26,6 +26,25 @@ off_t filesize(char *filename) {
 	}
 }
 
+/* Combine two strings to one with a '/' in the middle.
+ */
+char *make_path(char *dir, char *file) {
+	int dir_len, file_len;
+	char *path;
+
+	dir_len = strlen(dir);
+	file_len = strlen(file);
+
+	if ((path = (char*)malloc(dir_len + file_len + 2)) != NULL) {
+		memcpy(path, dir, dir_len);
+		path[dir_len] = '/';
+		memcpy(path + dir_len + 1, file, file_len);
+		path[dir_len + 1 + file_len] = '\0';
+	}
+
+	return path;
+}
+
 static enum t_fsbool outside_webroot(char *symlink, char *webroot) {
 	char filename[257], *slash;
 	int size, count;
@@ -120,8 +139,10 @@ enum t_fsbool is_directory(char *file) {
 			return no_access;
 		case ENOENT:
 			return not_found;
-		default:
+		case ENOTDIR:
 			return no;
+		default:
+			return error;
 	}
 }
 
@@ -201,7 +222,9 @@ void touch_logfile(char *dir, char *file, mode_t mode, uid_t uid, gid_t gid) {
 					fchown(fd, uid, gid);
 				}
 				close(fd);
-			}   
+			} else {
+				fprintf(stderr, "Warning: couldn't create logfile %s\n", logfile);
+			}
 		} else {
 			close(fd);
 		}   
@@ -361,13 +384,18 @@ t_filelist *read_filelist(char *directory) {
 			stat(filename, &status);
 
 			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
-				file->name = strdup(dir_info->d_name);
+				if ((file->name = strdup(dir_info->d_name)) == NULL) {
+					remove_filelist(filelist);
+					filelist = NULL;
+					break;
+				}
 				file->size = status.st_size;
 				file->time = status.st_mtime;
 				file->is_dir = S_ISDIR(status.st_mode);
 				file->next = filelist;
 			} else {
-				filelist = remove_filelist(filelist);
+				remove_filelist(filelist);
+				filelist = NULL;
 				break;
 			}
 			filelist = file;
@@ -416,15 +444,15 @@ t_filelist *sort_filelist(t_filelist *filelist) {
 
 /* free() a list of filenames.
  */
-t_filelist *remove_filelist(t_filelist *filelist) {
+void remove_filelist(t_filelist *filelist) {
 	t_filelist *file;
 
 	while (filelist != NULL) {
 		file = filelist;
 		filelist = filelist->next;
-		free(file->name);
+		if (file->name != NULL) {
+			free(file->name);
+		}
 		free(file);
 	}
-
-	return NULL;
 }
