@@ -43,7 +43,7 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 t_accesslist *remove_accesslist(t_accesslist *list);
 t_access ip_allowed(unsigned long ip, t_accesslist *list);
 
-t_keyvalue *parse_keyvaluelist(char *line, t_keyvalue *kvlist, char seperator);
+int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
 
 #endif
