@@ -1,28 +1,15 @@
-/*
- * This code implements the MD5 message-digest algorithm.
- * The algorithm is due to Ron Rivest.  This code was
- * written by Colin Plumb in 1993, no copyright is claimed.
- * This code is in the public domain; do with it what you wish.
- *
- * Equivalent code is available from RSA Data Security, Inc.
- * This code has been tested against that, and is equivalent,
- * except that you don't need to include two pages of legalese
- * with every copy.
- *
- * To compute the message digest of a chunk of bytes, declare an
- * MD5Context structure, pass it to MD5Init, call MD5Update as
- * needed on buffers full of bytes, and then call MD5Final, which
- * will fill a supplied 16-byte array with the digest.
- */
-
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
-#include "libmd5.h"
 
-/*
- * Note: this code is harmless on little-endian machines.
- */
+typedef unsigned long uint32;
+
+struct MD5Context {
+	uint32 buf[4];
+	uint32 bits[2];
+	unsigned char in[64];
+};
+
 static void byteReverse(unsigned char *buf, unsigned longs) {
 	uint32 t;
 
@@ -45,8 +32,7 @@ static void byteReverse(unsigned char *buf, unsigned longs) {
 /* This is the central step in the MD5 algorithm. */
 #define MD5STEP(f, w, x, y, z, data, s) ( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )
 
-/*
- * The core of the MD5 algorithm, this alters an existing MD5 hash to
+/* The core of the MD5 algorithm, this alters an existing MD5 hash to
  * reflect the addition of 16 longwords of new data.  MD5Update blocks
  * the data and converts bytes into longwords for this routine.
  */
@@ -132,8 +118,7 @@ static void MD5Transform(uint32 buf[4], uint32 const in[16]) {
 	buf[3] += d;
 }
 
-/*
- * Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
+/* Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
  * initialization constants.
  */
 static void MD5Init(struct MD5Context *ctx) {
@@ -146,15 +131,13 @@ static void MD5Init(struct MD5Context *ctx) {
 	ctx->bits[1] = 0;
 }
 
-/*
- * Update context to reflect the concatenation of another buffer full
+/* Update context to reflect the concatenation of another buffer full
  * of bytes.
  */
 static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned len) {
 	uint32 t;
 
 	/* Update bitcount */
-
 	t = ctx->bits[0];
 	if ((ctx->bits[0] = t + ((uint32) len << 3)) < t)
 		ctx->bits[1]++;		/* Carry from low to high */
@@ -163,7 +146,6 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
 	t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
 
 	/* Handle any leading odd-sized chunks */
-
 	if (t) {
 		unsigned char *p = (unsigned char *) ctx->in + t;
 
@@ -178,8 +160,8 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
 		buf += t;
 		len -= t;
 	}
-	/* Process data in 64-byte chunks */
 
+	/* Process data in 64-byte chunks */
 	while (len >= 64) {
 		memcpy(ctx->in, buf, 64);
 		byteReverse(ctx->in, 16);
@@ -189,12 +171,10 @@ static void MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned
 	}
 
 	/* Handle any remaining bytes of data. */
-
 	memcpy(ctx->in, buf, len);
 }
 
-/*
- * Final wrapup - pad to 64-byte boundary with the bit pattern 
+/* Final wrapup - pad to 64-byte boundary with the bit pattern 
  * 1 0* (64-bit count of bits processed, MSB-first)
  */
 static void MD5Final(unsigned char digest[16], struct MD5Context *ctx) {
@@ -234,7 +214,8 @@ static void MD5Final(unsigned char digest[16], struct MD5Context *ctx) {
 	MD5Transform(ctx->buf, (uint32 *) ctx->in);
 	byteReverse((unsigned char *) ctx->buf, 4);
 	memcpy(digest, ctx->buf, 16);
-	memset(ctx, 0, sizeof(ctx));	/* In case it's sensitive */
+	/* In case it's sensitive */
+	memset(ctx, 0, sizeof(ctx));
 }
 
 void md5_hash(char *string, int length, char hash[33]) {
