@@ -1,19 +1,22 @@
 #ifndef _LIBSSL_H
 #define _LIBSSL_H
 
+#include "config.h"
+
 #ifdef HAVE_SSL
 #include <stdbool.h>
-#include <openssl/err.h>
+#include <openssl/bio.h>
 #include <openssl/ssl.h>
+#include <openssl/err.h>
 
-void sslInit(void);
-SSL_CTX *ssl_binding(const char* sslKeyFile, const char* sslPassword);
-bool ssl_accept(int socketFD, SSL **ssl, SSL_CTX *sslContext);
+void ssl_init(void);
+SSL_CTX *ssl_binding(char *keyfile, char *CA_cert, int verify_depth, char *dh_file, char *ciphers);
+bool ssl_accept(int socketFD, SSL **ssl, SSL_CTX *context);
 int  ssl_receive(SSL *ssl, char *buffer, unsigned int maxlength);
 int  ssl_send(SSL *ssl, char *buffer, unsigned int length);
 int  ssl_close(SSL *ssl);
 void ssl_cleanup(SSL_CTX *sslContext);
-void ssl_set_environment(SSL *ssl_data);
+int  get_client_certificate(SSL *ssl_data, int size, char *subject, char *issuer);
 
 #endif
 
