@@ -3,11 +3,6 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
-#include <time.h>
-#include <dirent.h>
-#include <pthread.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 
 /* Convert a string to a integer.
  */
@@ -21,7 +16,7 @@ int str2int(const char *str) {
 	} else while (*(str + i) != '\0') {
 		if ((*(str + i) >= '0') && (*(str + i) <= '9')) {
 			value *= 10;
-			value += (*(str + i) - 48);
+			value += (*(str + i) - '0');
 			i++;
 		} else {
 			return -1;
@@ -33,24 +28,26 @@ int str2int(const char *str) {
 
 /* Remove the leading and trailing spaces in a string.
  */
-char* remove_spaces(char *str) {
+char *remove_spaces(char *str) {
 	int pos;
 
-	while ((*str == ' ') || (*str == '\t')) {
-		str++;
-	}
-	pos = strlen(str) - 1;
-	while (pos >= 0) {
-		switch (*(str + pos)) {
-			case ' ':
-			case '\t':
-			case '\r':
-			case '\n':
-				*(str + pos) = '\0';
-				pos--;
-				break;
-			default:
-				pos = -1;
+	if (str != NULL) {
+		while ((*str == ' ') || (*str == '\t')) {
+			str++;
+		}
+		pos = strlen(str) - 1;
+		while (pos >= 0) {
+			switch (*(str + pos)) {
+				case ' ':
+				case '\t':
+				case '\r':
+				case '\n':
+					*(str + pos) = '\0';
+					pos--;
+					break;
+				default:
+					pos = -1;
+			}
 		}
 	}
 
@@ -62,11 +59,15 @@ char* remove_spaces(char *str) {
 char *strip_line(char *str) {
 	char *hash;
 
-	if ((hash = strchr(str, '#')) != NULL) {
-		*hash = '\0';
+	if (str != NULL) {
+		if ((hash = strchr(str, '#')) != NULL) {
+			*hash = '\0';
+		}
+
+		return remove_spaces(str);
 	}
 
-	return remove_spaces(str);
+	return NULL;
 }
 
 /* Covert a string to lowercase.
@@ -74,12 +75,14 @@ char *strip_line(char *str) {
 char *strlower(char *line) {
 	char *c;
 
-	c = line;
-	while (*c != '\0') {
-		if ((*c >= 'A') && (*c <= 'Z')) {
-			*c += 32;
+	if (line != NULL) {
+		c = line;
+		while (*c != '\0') {
+			if ((*c >= 'A') && (*c <= 'Z')) {
+				*c += 32;
+			}
+			c++;
 		}
-		c++;
 	}
 
 	return line;
@@ -88,21 +91,50 @@ char *strlower(char *line) {
 /* Split a string in 2 strings.
  */
 int split_string(const char *str, char **key, char **value, char c) {
-	*key = (char*)str;
-	if ((*value = strchr(*key, c)) != NULL) {
-		*(*value)++ = '\0';
-		*key = remove_spaces(*key);
-		*value = remove_spaces(*value);
-		return 0;
-	} else {
-		return -1;
+	if (str != NULL) {
+		*key = (char*)str;
+		if ((*value = strchr(*key, c)) != NULL) {
+			*(*value)++ = '\0';
+			*key = remove_spaces(*key);
+			*value = remove_spaces(*value);
+
+			return 0;
+		}
 	}
+
+	return -1;
+}
+
+int split_configline(const char *str, char **key, char **value) {
+	*key = remove_spaces((char*)str);
+	*value = *key;
+	int eq = 0;
+
+	while (**value != '\0') {
+		if ((**value == ' ') || (**value == '=')) {
+			if (**value == '=') eq++;
+			**value = '\0';
+			do {
+				(*value)++;
+				if (**value == '=') eq++;
+			} while ((**value == ' ') || (**value == '='));
+
+			if (eq > 1) return -1;
+			return 0;
+		}
+		(*value)++;
+	}
+	value = NULL;
+
+	return -1;
 }
 
 /* Check the validity of an URL.
  */
 bool valid_uri(char *uri) {
-	if (*uri != '/') {
+	if (uri == NULL) {
+		return false;
+	} else if (*uri != '/') {
 		return false;
 	}
 
@@ -333,6 +365,10 @@ bool decode_base64(char *base64) {
 	const char *table = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	int l, i, t, byte = 0, bit = 0;
 
+	if (base64 == NULL) {
+		return false;
+	}
+
 	l = strlen(base64);
 	for (i = 0; i < l; i++) {
 		if (*(base64 + i) != '=') {
@@ -385,12 +421,14 @@ bool decode_base64(char *base64) {
 char *file_extension(char *filename) {
 	char *c;
 
-	c = filename + strlen(filename) - 1;
-	while (c >= filename) {
-		if (*c == '.') {
-			return c + 1;
+	if (filename != NULL) {
+		c = filename + strlen(filename) - 1;
+		while (c >= filename) {
+			if (*c == '.') {
+				return c + 1;
+			}
+			c--;
 		}
-		c--;
 	}
 
 	return NULL;
@@ -400,37 +438,23 @@ char *str_replace(char *line, char *from, char *to) {
 	char *pos, *new = NULL;
 	bool replaced = false;
 
-	while ((pos = strstr(line, from)) != NULL) {
-		if ((new = (char*)malloc(strlen(line) - strlen(from) + strlen(to) + 1)) == NULL) {
-			return NULL;
-		}
+	if ((line != NULL) && (from != NULL) && (to != NULL)) {
+		while ((pos = strstr(line, from)) != NULL) {
+			if ((new = (char*)malloc(strlen(line) - strlen(from) + strlen(to) + 1)) == NULL) {
+				return NULL;
+			}
 
-		*pos = '\0';
-		sprintf(new, "%s%s%s", line, to, pos + strlen(from));
+			*pos = '\0';
+			sprintf(new, "%s%s%s", line, to, pos + strlen(from));
 
-		if (replaced) {
-			free(line);
-		} else {
-			replaced = true;
+			if (replaced) {
+				free(line);
+			} else {
+				replaced = true;
+			}
+			line = new;
 		}
-		line = new;
 	}
 
 	return new;
 }
-
-#ifndef HAVE_STRCASESTR
-char *strcasestr(char *haystack, char *needle) {
-	int i, len, steps;
-
-	len = strlen(needle);
-	steps = strlen(haystack) - len + 1;
-	for (i = 0; i < steps; i++) {
-		if (strncasecmp(haystack + i, needle, len) == 0) {
-			return haystack + i;
-		}
-	}
-
-	return NULL;
-}
-#endif
