@@ -10,6 +10,7 @@ char *remove_spaces(char *str);
 char *strip_line(char *str);
 char *strlower(char *line);
 int split_string(const char *str, char **key, char **value, char c);
+int split_configline(const char *str, char **key, char **value);
 bool valid_uri(char *uri);
 int parse_special_chars(char *line, int length);
 int prevent_CMDi(char *line);
@@ -19,8 +20,5 @@ const char *errorstr(int code);
 bool decode_base64(char *base64);
 char *file_extension(char *filename);
 char *str_replace(char *line, char *from, char *to);
-#ifndef HAVE_STRCASESTR
-char *strcasestr(char *haystack, char *needle);
-#endif
 
 #endif
