@@ -4,13 +4,14 @@
 #include "session.h"
 
 void init_log_module(void);
-void log_string(char *logfile, char *data);
-void log_cgi_error(char *logfile, char *cgi_file, char *error);
-void log_warning(char *logfile, char *req, long ip_address);
+void log_pid(t_config *config, pid_t pid);
+void log_string(char *logfile, char *mesg);
+void log_system(t_session *session, char *mesg);
 void log_request(t_session *session, int code);
 void log_garbage(t_session *session);
 void log_unban(char *logfile, unsigned long ip_address, unsigned long connect_attempts);
-void log_close(t_host *host);
+void log_file_error(t_session *session, char *file, char *mesg);
 void log_error(t_session *session, char *mesg);
+void log_close(t_host *host);
 
 #endif
