@@ -19,6 +19,9 @@
 #include "libstr.h"
 #include "libmd5.h"
 #include "send.h"
+#ifdef HAVE_COMMAND
+#include "command.h"
+#endif
 
 #define MAX_CHUNK_SIZE 2048
 #define NONCE_DIGITS 10
@@ -97,6 +100,9 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			session->bytecounter += bytes_sent;
 		}
 	} while (total_sent < size);
+#ifdef HAVE_COMMAND
+	increment_transfer(TRANSFER_SEND, total_sent);
+#endif
 
 	return 0;
 }
