@@ -1,6 +1,8 @@
 #ifndef _SERVERCONFIG_H
 #define _SERVERCONFIG_H
 
+#include "config.h"
+#include <stdbool.h>
 #include <pthread.h>
 #include "mimetype.h"
 #include "libfs.h"
@@ -12,6 +14,7 @@
 
 typedef enum { root, part } t_pathmatch;
 typedef enum { basic, digest } t_auth_method;
+typedef enum { hiawatha, common, extended } t_log_format;
 
 typedef struct type_cgi_handler {
 	char          *handler;
@@ -40,6 +43,8 @@ typedef struct type_fcgi_server {
 	t_connect_to  *connect_to;
 	t_charlist    extension;
 	int           session_timeout;
+	char          *chroot;
+	size_t        chroot_len;
 
 	t_cgi_session *cgi_session_list[256];
 	pthread_mutex_t cgi_session_mutex[256];
@@ -62,6 +67,8 @@ typedef struct type_binding {
 	bool          use_ssl;
 	char          *server_key;
 	SSL_CTX       *ssl_context;
+	char          *ca_cert;
+	int           verify_depth;
 #endif
 
 	bool          enable_trace;
@@ -78,15 +85,17 @@ typedef struct type_binding {
 typedef struct type_directory {
 	char          *path;
 	t_pathmatch   path_match;
-	char          *cgi_wrap_id;
+	char          *wrap_cgi;
 	t_groups      groups;
 	char          *start_file;
 	bool          execute_cgi;
 	bool          execute_cgiset;
 	bool          show_index;
-	bool          show_indexSet;
+	bool          show_index_set;
 	bool          follow_symlinks;
-	bool          follow_symlinksSet;
+	bool          follow_symlinks_set;
+	bool          use_gz_file;
+	bool          use_gz_file_set;
 	char          *passwordfile;
 	t_auth_method auth_method;
 	char          *groupfile;
@@ -120,6 +129,8 @@ typedef struct type_host {
 	bool          user_websites;
 	bool          execute_cgi;
 	bool          show_index;
+	bool          use_gz_file;
+	bool          require_resolve_ip;
 #ifdef HAVE_PLUGIN 
 	bool          plugin_active;
 #endif
@@ -131,7 +142,7 @@ typedef struct type_host {
 	t_charlist    required_binding;
 	t_denybotlist *deny_bot;
 	t_charlist    required_group;
-	char          *cgi_wrap_id;
+	char          *wrap_cgi;
 	t_groups      groups;
 	t_charlist    volatile_object;
 	t_accesslist  *access_list;
@@ -161,13 +172,19 @@ typedef struct type_config {
 	t_groups      groups;
 	char          *server_string;
 	t_binding     *binding;
+	t_log_format  log_format;
 #ifdef HAVE_SSL
 	char          *server_key;
+	char          *ca_cert;
+	int           verify_depth;
+	char          *dh_file;
+	char          *allowed_ciphers;
 #endif
 	t_charlist    cgi_extension;
 	int           total_connections;
 	int           connections_per_ip;
 	int           time_for_cgi;
+	bool          kill_timedout_cgi;
 	char          *system_logfile;
 	char          *garbage_logfile;
 	char          *pidfile;
@@ -201,9 +218,9 @@ typedef struct type_config {
 	t_fcgi_server *fcgi_server;
 
 #ifdef HAVE_CACHE
-	int           cache_size;
-	int           cache_max_filesize;
-	int           cache_min_filesize;
+	off_t         cache_size;
+	off_t         cache_max_filesize;
+	off_t         cache_min_filesize;
 #endif
 
 #ifdef HAVE_COMMAND
@@ -212,10 +229,9 @@ typedef struct type_config {
 } t_config;
 
 t_config *default_config(void);
-void print_configerror(char *configfile, int code);
 int check_configuration(t_config *config);
-int read_main_configfile(char *file, t_config *config);
-int read_user_configfile(char *file, t_host *host);
+int read_main_configfile(char *configfile, t_config *config, bool config_check);
+int read_user_configfile(char *configfile, t_host *host);
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding);
 int read_throttleconfig(char *configfile, t_throttle **throttle);
 unsigned short get_throttlespeed(char *type, t_throttle *throttle);
