@@ -13,20 +13,11 @@
 #include <pwd.h>
 #include "libstr.h"
 #include "session.h"
-
-int new_client_id = 0;
-
 #ifdef HAVE_COMMAND
-extern unsigned long clients_served;
+#include "command.h"
 #endif
 
-char *reasons_for_403[5] = {
-	":filesystem",
-	":access_list",
-	":noCGIexec",
-	":hasSymlink",
-	":deny_bot"
-};
+int new_client_id = 0;
 
 /* Set the entries in a session-record to the default values.
  */
@@ -77,7 +68,7 @@ static void clear_session(t_session *session) {
  */
 void init_session(t_session *session) {
 #ifdef HAVE_COMMAND
-	clients_served++;
+	increment_counter(COUNTER_CLIENT);
 #endif
 	if ((session->client_id = new_client_id++) == MAX_CHILD_ID) {
 		new_client_id = 0;
@@ -91,8 +82,8 @@ void init_session(t_session *session) {
 	session->request = NULL;
 	session->buffer_size = 0;
 	session->bytes_in_buffer = 0;
-
 	session->flooding_timer = time(NULL);
+	session->ip_resolved = false;
 
 	clear_session(session);
 }
@@ -102,9 +93,9 @@ void init_session(t_session *session) {
 void reset_session(t_session *session) {
 	int size;
 
+	sfree(session->file_on_disk);
 	sfree(session->local_user);
 	sfree(session->remote_user);
-	sfree(session->file_on_disk);
 	remove_headerfields(session);
 	if (session->directory != NULL) {
 		pthread_mutex_lock(&(session->directory->client_mutex));
@@ -144,9 +135,6 @@ void reset_session(t_session *session) {
 	clear_session(session);
 }
 
-//void destroy_session(t_session *session) {
-//}
-
 /* Parse the HTTP headerfields from the received request.
  */
 t_headerfield *parse_headerfields(char *line) {
@@ -161,13 +149,17 @@ t_headerfield *parse_headerfields(char *line) {
 	while (*line != '\0') {
 		if (*line == '\r') {
 			*line = '\0';
-			if ((*(line + 1) == '\n') && (*(line + 2) != '\r') && (*(line + 2) != '\0')) {
-				if ((headerfield->next = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
-					return first;
+			if (*(line + 1) == '\n') {
+				if ((*(line + 2) != '\r') && (*(line + 2) != '\0')) {
+					if ((headerfield->next = (t_headerfield*)malloc(sizeof(t_headerfield))) == NULL) {
+						return first;
+					}
+					headerfield = headerfield->next;
+					headerfield->next = NULL;
+					headerfield->data = line + 2;
+				} else {
+					break;
 				}
-				headerfield = headerfield->next;
-				headerfield->next = NULL;
-				headerfield->data = line + 2;
 			}
 		}
 		line++;
@@ -189,12 +181,6 @@ t_headerfield *parse_headerfields(char *line) {
 	return first;
 }
 
-/* Set the reason for the 403 HTTP error
- */
-void set_403_reason(t_session *session, int reason) {
-	session->reason_for_403 = reasons_for_403[reason];
-}
-
 /* Search for a headerfield and return its value.
  */
 char *get_headerfield(char *key, t_headerfield *headerfields) {
@@ -230,14 +216,18 @@ void remove_headerfields(t_session *session) {
  */
 int get_homedir(t_session *session, char *username) {
 	struct passwd *pwd;
+	int len;
 
 	if ((pwd = getpwnam(username)) != NULL) {
-		if ((session->host->website_root = (char*)malloc(strlen(pwd->pw_dir) + 13)) == NULL) {
+		len = strlen(pwd->pw_dir);
+		if ((session->host->website_root = (char*)malloc(len + 13)) == NULL) {
 			return -1;
 		}
 
-		sprintf(session->host->website_root, "%s%s", pwd->pw_dir, "/public_html");
+		memcpy(session->host->website_root, pwd->pw_dir, len);
+		memcpy(session->host->website_root + len, "/public_html\0", 13);
 		session->websiteroot_copied = true;
+
 		return 200;
 	}
 
@@ -282,17 +272,15 @@ void restore_host(t_session *session) {
 /* Is the requested file marked as volatile?
  */
 bool is_volatile_object(t_session *session) {
-	bool retval = false;
 	int i;
 
 	for (i = 0; i < session->host->volatile_object.size; i++) {
 		if (strcmp(session->file_on_disk, *(session->host->volatile_object.item + i)) == 0) {
-			retval = true;
-			break;
+			return true;
 		}
 	}
 
-	return retval;
+	return false;
 }
 
 /* Copy the settings from a directory-record to the active host-record.
@@ -326,8 +314,8 @@ int copy_directory_settings(t_session *session) {
 							return 503;
 						}
 					}
-					if (dir->cgi_wrap_id != NULL) {
-						session->host->cgi_wrap_id = dir->cgi_wrap_id;
+					if (dir->wrap_cgi != NULL) {
+						session->host->wrap_cgi = dir->wrap_cgi;
 					}
 					if (dir->start_file != NULL) {
 						session->host->start_file = dir->start_file;
@@ -335,12 +323,15 @@ int copy_directory_settings(t_session *session) {
 					if (dir->execute_cgiset) {
 						session->host->execute_cgi = dir->execute_cgi;
 					}
-					if (dir->show_indexSet) {
+					if (dir->show_index_set) {
 						session->host->show_index = dir->show_index;
 					}
-					if (dir->follow_symlinksSet) {
+					if (dir->follow_symlinks_set) {
 						session->host->follow_symlinks = dir->follow_symlinks;
 					}
+					if (dir->use_gz_file_set) {
+						session->host->use_gz_file = dir->use_gz_file;
+					}
 					if (dir->access_list != NULL) {
 						session->host->access_list = dir->access_list;
 					}
