@@ -1,6 +1,7 @@
 #ifndef _SESSION_H
 #define _SESSION_H
 
+#include "config.h"
 #include <time.h>
 #ifdef HAVE_SSL
 #include "libssl.h"
@@ -12,12 +13,6 @@
 #define MAX_CHILD_ID 30000
 #define OUTPUT_BUFFER_SIZE 2000
 
-#define fb_FILESYSTEM 0
-#define fb_ACCESSLIST 1
-#define fb_NO_CGI     2
-#define fb_SYMLINK    3
-#define fb_DENYBOT    4
-
 typedef struct type_headerfield {
 	char *data;
 	int  value_offset;
@@ -63,9 +58,10 @@ typedef struct type_session {
 	char           *cookie;
 	bool           cookie_copied;
 	bool           websiteroot_copied;
-	long           bytes_sent;
+	off_t          bytes_sent;
 	int            return_code;
 	int            error_code;
+	bool           ip_resolved;
 
 	/* Throttling: send_buffer() in send.c
 	 */
@@ -93,7 +89,6 @@ typedef struct type_session {
 
 void init_session(t_session *session);
 void reset_session(t_session *session);
-//void destroy_session(t_session *session);
 
 void set_403_reason(t_session *session, int reason);
 t_headerfield *parse_headerfields(char *line);
