@@ -3,6 +3,7 @@
  * All the routines for sending the requested item back to the client.
  */
 #include "config.h"
+#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -15,13 +16,13 @@
 #include <pthread.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
 #ifdef HAVE_SENDFILE
 #include <sys/sendfile.h>
 #endif
+#include "alternative.h"
 #include "libstr.h"
 #include "libfs.h"
 #include "target.h"
@@ -31,6 +32,9 @@
 #ifdef HAVE_CACHE
 #include "cache.h"
 #endif
+#ifdef HAVE_COMMAND
+#include "command.h"
+#endif
 #ifdef HAVE_PLUGIN
 #include "plugin.h"
 #endif
@@ -39,31 +43,34 @@
 #define FILE_BUFFER_SIZE       32768
 #define MAX_CGI_HEADER          8192
 #define MAX_TRACE_HEADER        1024
+#define VALUE_SIZE                64
 
 #define FILESIZE_BUFFER_SIZE      30
-#define CGI_BUFFER_SIZE        65790  // fcgi.data + fcgi.padding
+#define CGI_BUFFER_SIZE        32768
+
+#define KILOBYTE        1024
+#define MEGABYTE     1048576
+#define GIGABYTE  1073741824
 
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
 #define rs_FORCE_QUIT -3
 
-char *chunked      = "Transfer-Encoding: chunked\r\n";
-char *hs_allow     = "Allow: GET, HEAD, OPTIONS";
-char *hs_post      = ", POST";
-char *hs_trace     = ", TRACE";
+char *chunked  = "Transfer-Encoding: chunked\r\n";
+char *hs_allow = "Allow: GET, HEAD, OPTIONS";
+char *hs_post  = ", POST";
+char *hs_trace = ", TRACE";
 
+extern char *fb_filesystem;
+extern char *fb_symlink;
 extern char *hs_eol;
 extern char *hs_conlen;
 extern char *hs_contyp;
 
-#ifdef HAVE_COMMAND
-extern unsigned long req_files, req_cgi, req_index;
-#endif
-
 /* Read a file from disk and send it to the client.
  */
 int send_file(t_session *session) {
-	char *referer, *buffer = NULL, value[64], *dot, *date, *range, *range_start, *range_end;
+	char *referer, *buffer = NULL, value[VALUE_SIZE + 1], *dot, *date, *range, *range_start, *range_end;
 	long bytes_read, total_bytes, size, speed;
 	off_t file_size, send_start, send_end, send_size;
 	int  retval, handle, http_code;
@@ -75,26 +82,42 @@ int send_file(t_session *session) {
 #endif
 
 #ifdef HAVE_COMMAND
-	req_files++;
+	increment_counter(COUNTER_FILE);
 #endif
 	http_code = 200;
 
 	session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
 
-	if (session->host->image_referer.size > 0) {
-		if (memcmp(session->mimetype, "image/", 6) == 0) {
-			if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
-				return 404;
-			} else {
-				close(handle);
+	handle = -1;
+	if (session->accept_gzip) {
+		size = strlen(session->file_on_disk);
+		memcpy(session->file_on_disk + size, ".gz\0", 4);
+		if ((handle = open(session->file_on_disk, O_RDONLY)) != -1) {
+			session->encode_gzip = true;
+		} else {
+			*(session->file_on_disk + size) = '\0';
+		}
+	}
+	if (handle == -1) {
+		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
+			if (errno == EACCES) {
+				log_error(session, fb_filesystem);
+				return 403;
 			}
+			return 404;
+		}
+	}
 
+	/* Check the referer
+	 */
+	if (session->host->image_referer.size > 0) {
+		if (strncmp(session->mimetype, "image/", 6) == 0) {
 			invalid_referer = true;
 			if ((referer = get_headerfield("Referer:", session->headerfields)) != NULL) {
-				if (memcmp(referer, "http://", 7) == 0) {
+				if (strncmp(referer, "http://", 7) == 0) {
 					prot_oke = true;
 					referer += 7;
-				} else if (memcmp(referer, "https://", 8) == 0) {
+				} else if (strncmp(referer, "https://", 8) == 0) {
 					prot_oke = true;
 					referer += 8;
 				} else {
@@ -124,36 +147,18 @@ int send_file(t_session *session) {
 			strcpy(session->file_on_disk, session->host->imgref_replacement);
 			session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
 		} else {
+			close(handle);
 			return 500;
 		}
 	}
 	
-	handle = -1;
-	if (session->accept_gzip) {
-		size = strlen(session->file_on_disk);
-		memcpy(session->file_on_disk + size, ".gz\0", 4);
-		if ((handle = open(session->file_on_disk, O_RDONLY)) != -1) {
-			session->encode_gzip = true;
-		} else {
-			*(session->file_on_disk + size) = '\0';
-		}
-	}
-	if (handle == -1) {
-		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
-			if (errno == EACCES) {
-				set_403_reason(session, fb_FILESYSTEM);
-				return 403;
-			}
-			return 404;
-		} else {
-		}
-	}
-
+	/* Symlink check
+	 */
 	if (session->host->follow_symlinks == false) {
 		switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
 			case error:
 				close(handle);
-				log_error(session, "target,send_file,symlinks");
+				log_error(session, "error while scanning file for symlinks");
 				return 500;
 			case not_found:
 				close(handle);
@@ -161,7 +166,7 @@ int send_file(t_session *session) {
 			case no_access:
 			case yes:
 				close(handle);
-				set_403_reason(session, fb_SYMLINK);
+				log_error(session, fb_symlink);
 				return 403;
 			case no:
 				break;
@@ -205,7 +210,7 @@ int send_file(t_session *session) {
 
 	if ((file_size = filesize(session->file_on_disk)) == -1) {
 		close(handle);
-		log_error(session, "target,send_file,filesize");
+		log_error(session, "error while determining filesize");
 		return 500;
 	}
 	send_start = 0;
@@ -216,7 +221,7 @@ int send_file(t_session *session) {
 	 */
 	if (*(session->http_version + 7) == '1') {
 		if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
-			if (memcmp(range, "bytes=", 6) == 0) {
+			if (strncmp(range, "bytes=", 6) == 0) {
 				range += 6;
 				if (split_string(range, &range_start, &range_end, '-') == 0) {
 
@@ -257,7 +262,17 @@ int send_file(t_session *session) {
 							close(handle);
 							return 416;
 						}
-					} else {
+					}
+
+					/* Change filepointer offset
+					 */
+					if (send_start > 0) {
+						if (lseek(handle, send_start, SEEK_SET) == -1) {
+							http_code = 200;
+						}
+					}
+
+					if (http_code == 200) {
 						send_start = 0;
 						send_end = file_size - 1;
 						send_size = file_size;
@@ -273,6 +288,8 @@ int send_file(t_session *session) {
 			break;
 		}
 
+		value[VALUE_SIZE] = '\0';
+
 		/* Last-Modified
 		 */
 		if (fstat(handle, &status) == -1) {
@@ -281,7 +298,7 @@ int send_file(t_session *session) {
 			break;
 		} else if (send_buffer(session, "Last-Modified: ", 15) == -1) {
 			break;
-		} else if (strftime(value, 64, "%a, %d %b %Y %X GMT\r\n", fdate) == 0) {
+		} else if (strftime(value, VALUE_SIZE, "%a, %d %b %Y %X GMT\r\n", fdate) == 0) {
 			break;
 		} else if (send_buffer(session, value, strlen(value)) == -1) {
 			break;
@@ -289,11 +306,10 @@ int send_file(t_session *session) {
 
 		/* Content-Range
 		 */
-		value[63] = '\0';
 		if (http_code == 206) {
 			if (send_buffer(session, "Content-Range: bytes ", 21) == -1) {
 				break;
-			} else if (snprintf(value, 63, "%lld-%lld/%lld\r\n", (long long)send_start, (long long)send_end, file_size) == -1) {
+			} else if (snprintf(value, VALUE_SIZE, "%lld-%lld/%lld\r\n", (long long)send_start, (long long)send_end, file_size) == -1) {
 				break;
 			} else if (send_buffer(session, value, strlen(value)) == -1) {
 				break;
@@ -302,7 +318,7 @@ int send_file(t_session *session) {
 
 		if (send_buffer(session, hs_conlen, 16) == -1) {
 			break;
-		} else if (snprintf(value, 63, "%lld\r\n\r\n", (long long)send_size) == -1) {
+		} else if (snprintf(value, VALUE_SIZE, "%lld\r\n\r\n", (long long)send_size) == -1) {
 			break;
 		} else if (send_buffer(session, value, strlen(value)) == -1) {
 			break;
@@ -310,9 +326,6 @@ int send_file(t_session *session) {
 
 		retval = 200;
 		if (session->head_request == false) {
-			if (send_start > 0) {
-				lseek(handle, send_start, SEEK_SET);
-			}
 			if (is_volatile_object(session) && (file_size <= MAX_VOLATILE_SIZE)) {
 				/* volatile object
 				 */
@@ -405,21 +418,22 @@ int send_file(t_session *session) {
 
 /* Run a CGI program and send output to the client.
  */
-int run_script(t_session *session) {
-	int retval = 200, result, handle, cgi_pid = -1;
+int execute_cgi(t_session *session) {
+	int retval = 200, result, handle;
 	char *line, *new_line, *contyp_start, *contyp_end, *code;
 	bool in_body = false, send_in_chunks = true, wrap_cgi, use_fast_cgi;
 	t_connect_to *connect_to;
 	t_cgi_info cgi_info;
+	pid_t cgi_pid = -1;
 
 #ifdef HAVE_COMMAND
-	req_cgi++;
+	increment_counter(COUNTER_CGI);
 #endif
 
 	if ((use_fast_cgi = (session->fcgi_server != NULL))) {
 		wrap_cgi = false;
 	} else {
-		wrap_cgi = (session->config->cgi_wrapper != NULL) && ((session->host->cgi_wrap_id != NULL) ||
+		wrap_cgi = (session->config->cgi_wrapper != NULL) && ((session->host->wrap_cgi != NULL) ||
 			((session->local_user != NULL) && session->config->wrap_user_cgi));
 	}
 
@@ -432,7 +446,7 @@ int run_script(t_session *session) {
 	if ((wrap_cgi == false) && (use_fast_cgi == false)) {
 		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
 			if (errno == EACCES) {
-				set_403_reason(session, fb_FILESYSTEM);
+				log_error(session, fb_filesystem);
 				return 403;
 			}
 			return 404;
@@ -442,7 +456,7 @@ int run_script(t_session *session) {
 	}
 
 	if (session->host->execute_cgi == false) {
-		set_403_reason(session, fb_NO_CGI);
+		log_error(session, "CGI execution not allowed");
 		return 403;
 	}
 
@@ -450,13 +464,13 @@ int run_script(t_session *session) {
 		if (session->cgi_handler == NULL) {
 			switch (can_execute(session->file_on_disk, session->config->server_uid, session->config->server_gid, &(session->config->groups))) {
 				case error:
-					log_error(session, "target,run_script,can_execute");
+					log_error(session, "error during CGI preprocess");
 					return 500;
 				case not_found:
 					return 404;
 				case no_access:
 				case no:
-					set_403_reason(session, fb_FILESYSTEM);
+					log_error(session, fb_filesystem);
 					return 403;
 				case yes:
 					break;
@@ -466,14 +480,13 @@ int run_script(t_session *session) {
 		if (session->host->follow_symlinks == false) {
 			switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
 				case error:
-					log_error(session, "target,run_script,symlink");
+					log_error(session, "error while searching for symlinks in CGI path");
 					return 500;
 				case not_found:
-printf("404-2\n");
 					return 404;
 				case no_access:
 				case yes:
-					set_403_reason(session, fb_SYMLINK);
+					log_error(session, fb_symlink);
 					return 403;
 				case no:
 					break;
@@ -493,7 +506,6 @@ printf("404-2\n");
 		if (session->vars != NULL) {
 			switch (prevent_SQLi(session->vars, strlen(session->vars) + 1, &line)) {
 				case -1:
-					log_error(session, "target,run_script,prevent_SQLi,URL");
 					return 500;
 				case 0:
 					break;
@@ -511,7 +523,6 @@ printf("404-2\n");
 			session->content_length = parse_special_chars(session->body, session->content_length);
 			switch (result = prevent_SQLi(session->body, session->content_length, &line)) {
 				case -1:
-					log_error(session, "target,run_script,prevent_SQLi,postdata");
 					return 500;
 				case 0:
 					break;
@@ -530,7 +541,6 @@ printf("404-2\n");
 			parse_special_chars(session->cookie, strlen(session->cookie));
 			switch (prevent_SQLi(session->cookie, strlen(session->cookie), &line)) {
 				case -1:
-					log_error(session, "target,run_script,preven_SQLi,cookie");
 					return 500;
 				case 0:
 					break;
@@ -561,25 +571,26 @@ printf("404-2\n");
 
 	cgi_info.input_buffer_size = cgi_info.error_buffer_size = CGI_BUFFER_SIZE;
 	cgi_info.input_len = cgi_info.error_len = 0;
-	cgi_info.wrap_cgi = wrap_cgi;
 
 	if (use_fast_cgi) {
+		cgi_info.read_header = true;
 		if ((connect_to = select_connect_to(session->fcgi_server, session->ip_address)) == NULL) {
 			return 503;
 		} else if ((cgi_info.from_cgi = connect_to_fcgi_server(connect_to)) == -1) {
 			connect_to->available = false;
-			log_string(session->config->system_logfile, "Can't connect to FastCGI server");
+			log_string(session->config->system_logfile, "can't connect to FastCGI server");
 			return 503;
 		} else {
 			connect_to->available = true;
-			if (send_fcgi_data(session, cgi_info.from_cgi) == -1) {
-				log_error(session, "target,run_script,send_fcgi_data");
+			if (send_fcgi_request(session, cgi_info.from_cgi) == -1) {
+				log_error(session, "error while sending data to FastCGI server");
 				return 500;
 			}
 		}
 	} else {
+		cgi_info.wrap_cgi = wrap_cgi;
 		if ((cgi_pid = fork_cgi_process(session, &cgi_info)) == -1) {
-			log_error(session, "target,run_script,fork_cgi_process");
+			log_error(session, "error while forking CGI process");
 			return 500;
 		}
 	}
@@ -614,170 +625,182 @@ printf("404-2\n");
 		}
 
 		switch (result) {
-			case cr_ERROR:
-			case cr_TIMEOUT:
+			case cgi_ERROR:
+				log_error(session, "error while executing CGI");
+				retval = 500;
+				break;
+			case cgi_TIMEOUT:
+				log_error(session, "CGI timeout");
 				if (in_body) {
 					retval = rs_DISCONNECT;
 				} else {
-					log_error(session, "target,run_script,in_body");
 					retval = 500;
 				}
+				if (session->config->kill_timedout_cgi && (use_fast_cgi == false)) {
+					kill(cgi_pid, 15);
+					sleep(1);
+					kill(cgi_pid, 9);
+				}
 				break;
-			case cr_FORCE_QUIT:
+			case cgi_FORCE_QUIT:
 				retval = rs_FORCE_QUIT;
 				break;
-			case cr_OKE:
-				switch (cgi_info.input_result) {
-					case cd_READ_DATA:
-						/* Data received from CGI
+			case cgi_OKE:
+				if (cgi_info.input_len > 0) {
+					/* Data received from CGI
+					 */
+					if (in_body) {
+						/* Read content
 						 */
-						if (in_body) {
-							/* Read content
-							 */
-							if (session->head_request == false) {
-								if (send_in_chunks) {
-									if (send_chunk(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-										retval = rs_DISCONNECT;
-									}
-								} else {
-									if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-										retval = rs_DISCONNECT;
+						if (session->head_request == false) {
+							if (send_in_chunks) {
+								if (send_chunk(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
+									retval = rs_DISCONNECT;
+								}
+							} else {
+								if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
+									retval = rs_DISCONNECT;
+								}
+							}
+						}
+						cgi_info.input_len = 0;
+					} else {
+						/* Read HTTP header
+						 */
+						*(cgi_info.input_buffer + cgi_info.input_len) = '\0';
+
+						if ((new_line = strstr(cgi_info.input_buffer, "\r\n\r\n")) != NULL) {
+							if (session->throttle == 0) {
+								if ((contyp_start = strcasestr(cgi_info.input_buffer, hs_contyp)) != NULL) {
+									if ((contyp_end = strchr(contyp_start, '\r')) != NULL) {
+										contyp_start += 14;
+										*contyp_end = '\0';
+										session->throttle = get_throttlespeed(contyp_start, session->config->throttle);
+										*contyp_end = '\r';
 									}
 								}
 							}
-							cgi_info.input_len = 0;
-						} else {
-							/* Read HTTP header
-							 */
-							*(cgi_info.input_buffer + cgi_info.input_len) = '\0';
 
-							if ((new_line = strstr(cgi_info.input_buffer, "\r\n\r\n")) != NULL) {
-								if (session->throttle == 0) {
-									if ((contyp_start = strcasestr(cgi_info.input_buffer, hs_contyp)) != NULL) {
-										if ((contyp_end = strchr(contyp_start, '\r')) != NULL) {
-											contyp_start += 14;
-											*contyp_end = '\0';
-											session->throttle = get_throttlespeed(contyp_start, session->config->throttle);
-											*contyp_end = '\r';
-										}
-									}
-								}
-
-								if (strcasestr(cgi_info.input_buffer, "Location:") != NULL) {
-									if (send_header(session, 302) == -1) {
-										retval = rs_DISCONNECT;
-										break;
-									}
-								} else if ((code = strcasestr(cgi_info.input_buffer, "Status: ")) != NULL) {
-									line = code += 8;
-									while (*line != '\0') {
-										if (*line == '\r') {
-											*line = '\0';
-										}
-										line++;
-									}
-									if ((retval = atoi(code)) == 0) {
-										log_error(session, "target,run_script,statuscode");
-										retval = 500;
-									}
-									if (send_header(session, retval) == -1) {
-										retval = rs_DISCONNECT;
-									}
-									break;
-								} else if (send_header(session, session->return_code) == -1) {
+							if (strcasestr(cgi_info.input_buffer, "Location:") != NULL) {
+								if (send_header(session, 302) == -1) {
 									retval = rs_DISCONNECT;
 									break;
 								}
-								if (session->return_code == 401) {
-									if (session->host->auth_method == basic) {
-										send_basic_auth(session);
-									} else {
-										send_digest_auth(session);
+							} else if ((code = strcasestr(cgi_info.input_buffer, "Status: ")) != NULL) {
+								line = code += 8;
+								while (*line != '\0') {
+									if (*line == '\r') {
+										*line = '\0';
 									}
+									line++;
 								}
-								if ((strcasestr(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
-									send_in_chunks = false;
-								} else if (send_buffer(session, chunked, 28) == -1) {
+								if ((retval = str2int(code)) <= 0) {
+									log_error(session, "invalid status code received from CGI");
+									retval = 500;
+								}
+								if (retval == 403) {
+									log_error(session, "CGI-wrapper returned 403");
+								}
+								if (send_header(session, retval) == -1) {
 									retval = rs_DISCONNECT;
-									break;
 								}
-
-								/* Send the header.
-								 */
-								if (send_in_chunks || session->head_request) {
-									new_line += 4;
-									result = new_line - cgi_info.input_buffer;
-									if (send_buffer(session, cgi_info.input_buffer, result) == -1) {
-										retval = rs_DISCONNECT;
-										break;
-									}
-									if (send_buffer(session, NULL, 0) == -1) {
-										retval = rs_DISCONNECT;
-										break;
-									}
-									if (session->head_request == false) {
-										result = cgi_info.input_len - result;
-										if (result > 0) {
-											if (send_chunk(session, new_line, result) == -1) {
-												retval = rs_DISCONNECT;
-												break;
-											}
-										}
-									} else {
-										//retval = rs_QUIT;
-										//break;
-									}
-								} else {
-									if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-										retval = rs_DISCONNECT;
-										break;
-									}
-								}
-
-								in_body = true;
-								cgi_info.input_len = 0;
-							} else if (cgi_info.input_len > MAX_CGI_HEADER) {
-								retval = 413;
+								break;
+							} else if (send_header(session, session->return_code) == -1) {
+								retval = rs_DISCONNECT;
 								break;
 							}
-						}
-						break;
-					case cd_END_OF_DATA:
-						if (in_body) {
-							retval = rs_QUIT;
-							if (send_in_chunks) {
-								if (send_chunk(session, NULL, 0) == -1) {
-									retval = rs_DISCONNECT;
+							if (session->return_code == 401) {
+								if (session->host->auth_method == basic) {
+									send_basic_auth(session);
+								} else {
+									send_digest_auth(session);
 								}
 							}
-						} else {
-							log_error(session, "target,run_script,cd_END_OF_DATA");
-							retval = 500;
-						}
-						break;
-					case cd_NOTHING:
-						break;
-					case cd_READ_ERROR:
-						retval = rs_DISCONNECT;
-						break;
-				} // switch
+							if ((strcasestr(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
+								send_in_chunks = false;
+							} else if (send_buffer(session, chunked, 28) == -1) {
+								retval = rs_DISCONNECT;
+								break;
+							}
 
-				switch (cgi_info.error_result) {
-					case cd_READ_DATA:
-						*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
-						log_cgi_error(session->host->error_logfile, session->file_on_disk, cgi_info.error_buffer);
-						cgi_info.error_len = 0;
-						break;
-					case cd_END_OF_DATA:
-						break;
-					case cd_NOTHING:
-						break;
-					case cd_READ_ERROR:
-						break;
-				} // switch
+							/* Send the header.
+							 */
+							if (send_in_chunks || session->head_request) {
+								new_line += 4;
+								result = new_line - cgi_info.input_buffer;
+								if (send_buffer(session, cgi_info.input_buffer, result) == -1) {
+									retval = rs_DISCONNECT;
+									break;
+								}
+								if (send_buffer(session, NULL, 0) == -1) {
+									retval = rs_DISCONNECT;
+									break;
+								}
+								if (session->head_request == false) {
+									result = cgi_info.input_len - result;
+									if (result > 0) {
+										if (send_chunk(session, new_line, result) == -1) {
+											retval = rs_DISCONNECT;
+											break;
+										}
+									}
+								} else {
+									//retval = rs_QUIT;
+									//break;
+								}
+							} else {
+								if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
+									retval = rs_DISCONNECT;
+									break;
+								}
+							}
+
+							in_body = true;
+							cgi_info.input_len = 0;
+						} else if (cgi_info.input_len > MAX_CGI_HEADER) {
+							retval = 413;
+							break;
+						}
+					}
+				}
+
+				if (cgi_info.error_len > 0) {
+					/* Error received from CGI
+					 */
+					*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
+					log_file_error(session, session->file_on_disk, cgi_info.error_buffer);
+					cgi_info.error_len = 0;
+				}
+				break;
+			case cgi_END_OF_DATA:
+				if (in_body) {
+					retval = rs_QUIT;
+					if (send_in_chunks) {
+						if (send_chunk(session, NULL, 0) == -1) {
+							retval = rs_DISCONNECT;
+						}
+					}
+				} else {
+					if (cgi_info.input_len == 0) {
+						log_file_error(session, session->file_on_disk, "no output");
+					} else if (strstr(cgi_info.input_buffer, "\r\n\r\n") == NULL) {
+						log_file_error(session, session->file_on_disk, "non-RFC HTTP header");
+					} else {
+						log_error(session, "CGI only printed a HTTP header, no content");
+					}
+					retval = 500;
+				}
 		} // switch
 	} while (retval == 200);
 
+	if (use_fast_cgi) {
+		close(cgi_info.from_cgi);
+	} else {
+		close(cgi_info.to_cgi);
+		close(cgi_info.from_cgi);
+		close(cgi_info.cgi_error);
+	}
+
 	switch (retval) {
 		case rs_DISCONNECT:
 		case rs_FORCE_QUIT:
@@ -786,16 +809,6 @@ printf("404-2\n");
 			retval = 200;
 	}
 
-	if (use_fast_cgi) {
-		close(cgi_info.from_cgi);
-	} else {
-		kill(cgi_pid, 9);
-
-		close(cgi_info.to_cgi);
-		close(cgi_info.from_cgi);
-		close(cgi_info.cgi_error);
-	}
-
 	free(cgi_info.input_buffer);
 	free(cgi_info.error_buffer);
 
@@ -805,29 +818,20 @@ printf("404-2\n");
 /* Converts a filesize to a string.
  */
 static int filesize2str(char *buffer, off_t fsize) {
-	//unsigned long long fsize;
 	int result = 0;
 
-	//fsize = (unsigned long)size;
-
 	buffer[FILESIZE_BUFFER_SIZE - 1] = '\0';
-	if (fsize < 1024) {
+	if (fsize < KILOBYTE) {
 		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%llu byte", (long long)fsize);
-	} else if (fsize < 1048576) {
+	} else if (fsize < MEGABYTE) {
 		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f kB", ((double)(fsize >> 6)) / 16);
-	} else if (fsize < 1073741824) {
+	} else if (fsize < GIGABYTE) {
 		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f MB", ((double)(fsize >> 16)) / 16);
 	} else {
 		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f GB", ((double)(fsize >> 26)) / 16);
 	}
 
 	return (result < 0) ? 0 : result;
-/*
-	} else {
-		sprintf(buffer, "> 4 GB");
-		return 6;
-	}
-*/
 }
 
 /* Read the content of a directory and send it in a formatted list to the client.
@@ -837,11 +841,12 @@ int show_index(t_session *session) {
 	off_t total_fsize = 0;
 	char size[FILESIZE_BUFFER_SIZE], color = '0', timestr[33], *slash;
 	struct tm s;
-	t_filelist *filelist = NULL;
+	t_filelist *filelist = NULL, *file;
+	t_keyvalue *alias;
 	bool root_dir;
 
 #ifdef HAVE_COMMAND
-	req_index++;
+	increment_counter(COUNTER_INDEX);
 #endif
 
 	session->mimetype = NULL;
@@ -858,7 +863,7 @@ int show_index(t_session *session) {
 		case not_found:
 			return 404;
 		case no_access:
-			set_403_reason(session, fb_FILESYSTEM);
+			log_error(session, fb_filesystem);
 			return 403;
 			break;
 		case yes:
@@ -873,7 +878,7 @@ int show_index(t_session *session) {
 				return 404;
 			case no_access:
 			case yes:
-				set_403_reason(session, fb_SYMLINK);
+				log_error(session, fb_symlink);
 				return 403;
 			case no:
 				break;
@@ -895,10 +900,34 @@ int show_index(t_session *session) {
 		return 200;
 	}
 
+	/* Read directory content
+	 */
 	if ((filelist = read_filelist(session->file_on_disk)) == NULL) {
 		return 500;
 	}
-	filelist = sort_filelist(filelist);
+
+	/* Add aliasses to directory list
+	 */
+	alias = session->host->alias;
+	while (alias != NULL) {
+		if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
+			if ((file->name = strdup(alias->key + 1)) == NULL) {
+				remove_filelist(filelist);
+				return 500;
+			}
+			file->size = 0;
+			file->time = time(NULL);
+			file->is_dir = true;
+			file->next = filelist;
+			filelist = file;
+		} else {
+			remove_filelist(filelist);
+			return 500;
+		}
+		alias = alias->next;
+	}
+
+	file = filelist = sort_filelist(filelist);
 
 	do {
 		if (session->keep_alive) {
@@ -946,10 +975,10 @@ int show_index(t_session *session) {
 
 		root_dir = (strcmp(session->uri, "/") == 0);
 
-		while (filelist != NULL) {
-			if (filelist->is_dir && root_dir) {
-				if (strcmp(filelist->name, "..") == 0) {
-					filelist = filelist->next;
+		while (file != NULL) {
+			if (file->is_dir && root_dir) {
+				if (strcmp(file->name, "..") == 0) {
+					file = file->next;
 					continue;
 				}
 			}
@@ -961,29 +990,27 @@ int show_index(t_session *session) {
 			} else if (send_chunk(session, "\"><td class=\"left\">", 19) == -1) {
 				break;
 			}
-			if (filelist->is_dir) {
+			if (file->is_dir) {
 				if (send_chunk(session, "[dir]", 5) == -1) {
 					break;
 				}
 			}
 			if (send_chunk(session, "</td><td class=\"filename\"><a href=\"", 35) == -1) {
 				break;
-			} else if (send_chunk(session, session->uri, strlen(session->uri)) == -1) {
-				break;
-			} else if (send_chunk(session, filelist->name, strlen(filelist->name)) == -1) {
+			} else if (send_chunk(session, file->name, strlen(file->name)) == -1) {
 				break;
 			}
-			if (filelist->is_dir) {
+			if (file->is_dir) {
 				if (send_chunk(session, "/", 1) == -1) {
 					break;
 				}
 			}
 			if (send_chunk(session, "\">", 2) == -1) {
 				break;
-			} else if (send_chunk(session, filelist->name, strlen(filelist->name)) == -1) {
+			} else if (send_chunk(session, file->name, strlen(file->name)) == -1) {
 				break;
 			}
-			if (filelist->is_dir) {
+			if (file->is_dir) {
 				if (send_chunk(session, "/", 1) == -1) {
 					break;
 				}
@@ -991,7 +1018,7 @@ int show_index(t_session *session) {
 			if (send_chunk(session, "</a></td><td class=\"date\">", 26) == -1) {
 				break;
 			}
-			s = *localtime(&(filelist->time));
+			s = *localtime(&(file->time));
 			*(timestr + 32) = '\0';
 			strftime(timestr, 32, "%d %b %Y, %X", &s);
 			if (send_chunk(session, timestr, strlen(timestr)) == -1) {
@@ -999,8 +1026,8 @@ int show_index(t_session *session) {
 			} else if (send_chunk(session, "</td><td class=\"size\">", 22) == -1) {
 				break;
 			}
-			if (filelist->is_dir == false) {
-				len = filesize2str(size, filelist->size);
+			if (file->is_dir == false) {
+				len = filesize2str(size, file->size);
 				if (send_chunk(session, size, len) == -1) {
 					break;
 				}
@@ -1011,10 +1038,12 @@ int show_index(t_session *session) {
 			color = '0' + (1 - (color - '0'));
 
 			total_files++;
-			total_fsize += filelist->size;
-			filelist = filelist->next;
+			if (file->is_dir == false) {
+				total_fsize += file->size;
+			}
+			file = file->next;
 		}
-		if (filelist != NULL) {
+		if (file != NULL) {
 			break;
 		}
 
@@ -1058,7 +1087,7 @@ int handle_options_request(t_session *session) {
 			code = 404;
 		}
 		if (errno == EACCES) {
-			set_403_reason(session, fb_FILESYSTEM);
+			log_error(session, fb_filesystem);
 			code = 403;
 		}
 	} else {
@@ -1074,7 +1103,7 @@ int handle_options_request(t_session *session) {
 				break;
 			case no_access:
 			case yes:
-				set_403_reason(session, fb_SYMLINK);
+				log_error(session, fb_symlink);
 				code = 403;
 				break;
 			case no:
@@ -1110,7 +1139,7 @@ int handle_options_request(t_session *session) {
 int handle_trace_request(t_session *session) {
 	int result = -1, code, body_size;
 	size_t len;
-	char buffer[MAX_TRACE_HEADER + 2];
+	char buffer[MAX_TRACE_HEADER + 1];
 	t_headerfield *header;
 
 	body_size = 3;
@@ -1126,7 +1155,7 @@ int handle_trace_request(t_session *session) {
 		header = header->next;
 	}
 
-	buffer[MAX_TRACE_HEADER + 1] = '\0';
+	buffer[MAX_TRACE_HEADER] = '\0';
 
 	do {
 		// Header
@@ -1190,7 +1219,10 @@ int handle_plugin(t_session *session) {
 
 	if ((request_info = (t_request_info*)malloc(sizeof(t_request_info))) != NULL) {
 		request_info->fd = session->client_socket;
-		request_info->fp = fdopen(session->client_socket, "w");
+		if ((request_info->fp = fdopen(session->client_socket, "w")) == NULL) {
+			free(request_info);
+			return 500;
+		}
 		request_info->keep_alive = session->keep_alive;
 		request_info->parameters = session->vars;
 		request_info->cookies = get_headerfield("Cookie:", session->headerfields);
