@@ -7,7 +7,7 @@
 #define rr_CMD_INJECTION -51
 
 int send_file(t_session *session);
-int run_script(t_session *session);
+int execute_cgi(t_session *session);
 int show_index(t_session *session);
 int handle_options_request(t_session *session);
 int handle_trace_request(t_session *session);
